function map = GenerateHeightMap()

  CONSTANTS

  [lat,lon] = meshgrid(LAT_RANGE, LON_RANGE);
  lat = lat(:);
  lon = lon(:);

  grid_data = GetSRTMData([lon,lat]);

  lon = reshape(grid_data(:,1), GRID_SIZE);
  lat = reshape(grid_data(:,2), GRID_SIZE);
  height = reshape(grid_data(:,3), GRID_SIZE);
  old_height = flipud(height');

  dead_mask = height > 10000;
  kernel = [0,1,0; 1,1,1; 0,1,0]>0;

  while any(dead_mask(:))

    fringe_mask = dead_mask & ~imerode(dead_mask, kernel);
    [fringe_x, fringe_y] = find(fringe_mask);
    new_height = height;
    for fringe_i = 1:size(fringe_x,1)
      x = fringe_x(fringe_i);
      y = fringe_y(fringe_i);
      temp_mask = false(size(fringe_mask));
      temp_mask(x,y) = true;
      temp_mask = imdilate(temp_mask, kernel) & ~dead_mask;
      new_height(x,y) = mean(height(temp_mask));
    end
    height = new_height;
    dead_mask = height > 10000;

  end

  height = imfilter(height, fspecial('gaussian', 10, 1), 'same', 'replicate');

  map.lon = flipud(lon');
  map.lat = flipud(lat');
  map.height = flipud(height');

  map.lat = map.lat(:,1);
  map.lon = map.lon(1,:)';
  
  if nargout == 0
    DrawHeightMap(map);
  end
  
end


