% Copyright (c) 2008, Rob Fergus and Jon Barron (NYU)
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
%     * Redistributions of source code must retain the above copyright
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright
%       notice, this list of conditions and the following disclaimer in the
%       documentation and/or other materials provided with the
%       distribution.
%     * Neither the name of NYU nor the
%       names of its contributors may be used to endorse or promote products
%       derived from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY ROB FERGUS AND JON BARRON ``AS IS'' AND ANY
% EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
% DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function model=backprop_DrLIM(model,data,NUM_EPOCHS, soft_alpha, n_soft, MAX_ITS)
 
  DEBUG = 0;
  
  if (nargin==2)
    NUM_EPOCHS = 3; % default # epochs
  end
  if (nargin<=3)
      soft_alpha = 1;
  end
  if (nargin<=4)
      n_soft = 1;
  end
  if (nargin<=5)
    MAX_ITS = 3;
  end
  
  M_DrLIM = model.drlim.m;
  
  
  if DEBUG
  
    debug_limit = [416,10,10,10,10,416];
    
    % Init weights
    % stack the weights together
    for a=1:model.numlayers
      model.backprop.weights{a} = single([model.layer(a).vishid(1:debug_limit(a),1:debug_limit(a+1))]);
      model.backprop.bias{a} = single([model.layer(a).hidbiases(1:debug_limit(a+1))]);
      model.backprop.dimensions(a,:) = size(model.backprop.weights{a}) + [1 0];
    end
  
    for a=model.numlayers:-1:1
      model.backprop.weights{2*model.numlayers-a+1} = single([model.layer(a).vishid(1:debug_limit(a),1:debug_limit(a+1))']);
      model.backprop.bias{2*model.numlayers-a+1} = single([model.layer(a).visbiases(1:debug_limit(a))]);
      model.backprop.dimensions(2*model.numlayers-a+1,:) = size(model.backprop.weights{2*model.numlayers-a+1}) + [1 0];
    end
   
  else

    if ~isfield(model,'backprop')
      model.backprop.scores = {};
      for a=1:model.numlayers
        model.backprop.weights{a} = single([model.layer(a).vishid]);
        model.backprop.bias{a} = single([model.layer(a).hidbiases]);
        model.backprop.dimensions(a,:) = size(model.backprop.weights{a}) + [1 0];
      end
      
      for a=model.numlayers:-1:1
        model.backprop.weights{2*model.numlayers-a+1} = single([model.layer(a).vishid']);
        model.backprop.bias{2*model.numlayers-a+1} = single([model.layer(a).visbiases]);
        model.backprop.dimensions(2*model.numlayers-a+1,:) = size(model.backprop.weights{2*model.numlayers-a+1}) + [1 0];
      end
    else
%       fprintf('Using existing backprop\n');
    end
    
  end

for n = [(model.numlayers-(n_soft-1)):model.numlayers]
    model.backprop.weights{n} = model.backprop.weights{n}/soft_alpha;
    model.backprop.bias{n} = model.backprop.bias{n}/soft_alpha;
end

 
      % vectorize weights in network
      weight_vec = [];
      for a=1:model.numlayers*2
        tmp = [model.backprop.weights{a} ; model.backprop.bias{a}];
        weight_vec = [ weight_vec ; tmp(:)];
      end
      
     ss = prod(model.backprop.dimensions,2);
  
     if DEBUG
      
       global bitDiff_start
       bitDiff_start = [];
       
       %%% Check gradient routine
       model.drlim.noise_inject = model.drlim.noise_inject(1:end,1:10);
       [q,a,n]=checkgrad('CG_DrLIM',double(weight_vec(1:sum(ss(1:model.numlayers)))),1e-4,model.backprop.dimensions(1:model.numlayers,:),double(data.pairs(1:end,1:debug_limit(end))), data.neighbors(1:end), model.arch.hidbias_offset,model.arch.hidlinear,model.arch.vislinear, model.drlim);
       figure; plot(a,'r'); hold on; plot(n,'b'); legend('Analytic','Numerical');
       keyboard
       % q is difference, a is analytic, n is numerical
           
     else
       [weight_vec_out,fX, fX_pull, fX_push, model.drlim] = minimize_Stoch(double(weight_vec(1:sum(ss(1:model.numlayers)))),'CG_DrLIM', 1,model.backprop.dimensions(1:model.numlayers,:),double(data.pairs), data.neighbors , model.arch.hidbias_offset,model.arch.hidlinear,model.arch.vislinear, model.drlim);
     end
     
      weight_vec_out = [ weight_vec_out ; weight_vec(length(weight_vec_out)+1:end)];
      score = [fX_pull, fX_push];
  
      %%% decant vector of weights into matrices
      for i=1:model.numlayers*2
        
        if (i==1)
          prev_dims=0; 
        else
          prev_dims = sum(prod( model.backprop.dimensions(1:i-1,:),2));
        end
        
        tmp = weight_vec_out(prev_dims+1:prev_dims+prod( model.backprop.dimensions(i,:)));
        tmp = reshape(tmp, model.backprop.dimensions(i,:));
        model.backprop.weights{i} = tmp(1:end-1,:);
        
      end
   
   model.backprop.scores{length(model.backprop.scores)+1} = score;
  
   % % save model
%    save(['/tmp/backprop_drlim_',num2str(model.arch.numhid(end)),'_',num2str(round(model.layer(1).randseed(1)*1000))],'model');

  
