function [accelerations] = correctAccelerations(accelerations, biker, path, is_start, is_final)

  CONSTANTS

  distances = path.true_dists;
  grades = path.grades;
  
%   accelerations = max(1e-3, accelerations);
  
  len = length(accelerations);
  
  if nargin < 5
    is_final = 0;
  end
  
  % Reset accelerations such that we never fall down/fail
  ad = zeros(len,1);
  ad_sum = 0;
  min_ad = ((biker.min_speed/sqrt(2)).^2) - ~is_start*(0.5*biker.stop_speed^2);
  max_ad = ((biker.max_speed/sqrt(2)).^2) - ~is_start*(0.5*biker.stop_speed^2);
  stop_ad = ((biker.stop_speed/sqrt(2)).^2);
  
  for i = 1:len
    
    % ad_sum must be greater than zero
    if (ad_sum + accelerations(i) * distances(i)) < min_ad
      accelerations(i) = (min_ad - ad_sum)/distances(i);
    end
    
    % ad_sum must be less than some speed
    if (ad_sum + accelerations(i) * distances(i)) > max_ad
      accelerations(i) = (max_ad - ad_sum)/distances(i);
    end
    
    % cap the final speed, if requested
    if i == len && is_final
      if (ad_sum + accelerations(i) * distances(i)) > stop_ad
        accelerations(i) = (stop_ad - ad_sum)/distances(i);
      end
    end
    
    ad_sum = ad_sum + accelerations(i) * distances(i);
    
  end
  
end
