function [control_points, control_vecs] = findControlPoints(nodes_tree, nodes_tree_warp, edges_tree, nodes_mesh, tris)

control_points = {};
control_vecs = {};

for ni = 1:size(nodes_mesh,1)

  N = nodes_mesh(ni,:);
  nio = ni + size(nodes_tree,1);

  T = tris(any(tris == nio,2),:);

  nodes_tree_adjacent = unique(T(T<=size(nodes_tree,1)));

  edges_tree_adjacent = edges_tree(any(ismember(edges_tree, nodes_tree_adjacent),2),:);

  % iterate, record distance to that edge, and if the node lies within the edge
  projs = zeros(size(edges_tree_adjacent,1),1);
  dists = zeros(size(edges_tree_adjacent,1),1);
  flips = zeros(size(edges_tree_adjacent,1),1);
  for ei = 1:size(edges_tree_adjacent,1)
    E = edges_tree_adjacent(ei,:);

    N1 = nodes_tree(E(1),:);
    N2 = nodes_tree(E(2),:);

    N_vec = N - N1;
    E_vec = N2 - N1;

    proj = (N_vec * E_vec') ./ sum(E_vec.^2);

    N_on_E = proj * E_vec + N1;

    vec = N_on_E - N1;
    vec = vec ./ (eps+sqrt(sum(vec.^2)));

    vec_rot = N - N_on_E;
    vec_rot = vec_rot ./ (eps+sqrt(sum(vec_rot.^2)));

    flips(ei) = vec_rot * [-vec(2), vec(1)]' < 0;

    projs(ei) = proj;
    dists(ei) = sqrt(sum((N - N_on_E).^2));

  end

  proj_valid = (projs >= 0) & (projs <= 1);

  if any( proj_valid )

    idx = find(proj_valid);
    take = idx(find(dists(idx) == min(dists(idx)), 1, 'first'));
    E = edges_tree_adjacent(take,:);
    proj = projs(take);

    N1 = nodes_tree_warp(E(1),:);
    N2 = nodes_tree_warp(E(2),:);

    E_vec = N2 - N1;

    N_on_E = E_vec * proj + N1;

    vec = N_on_E - N1;
    vec = vec ./ (eps+sqrt(sum(vec.^2)));

    vec_rot = [-vec(2), vec(1)];

    if flips(take)
      vec_rot = -vec_rot;
    end

    control_points{ni} = N_on_E;
    control_vecs{ni} = vec_rot;

  else

    assert(nnz(nodes_tree_adjacent) == 1)

    root_tree_node_idx = nodes_tree_adjacent;
    other_tree_node_idx = setdiff(edges_tree_adjacent(:), root_tree_node_idx);

    if length(other_tree_node_idx) == 1

      Nr = nodes_tree(root_tree_node_idx,:);
      No = nodes_tree(other_tree_node_idx,:);

      N_vec = N - Nr;
      No_vec = No - Nr;

      [t] = cart2pol(N_vec(:,1), N_vec(:,2));
      [to] = cart2pol(No_vec(:,1), No_vec(:,2));
      dt = thetaDist(t, to);

      point = Nr;
      vec = N_vec ./ sqrt(sum(N_vec.^2));


      Nr_warp = nodes_tree_warp(root_tree_node_idx,:);
      No_warp = nodes_tree_warp(other_tree_node_idx,:);

      No_warp_vec = No_warp - Nr_warp;

      [to_warp] = cart2pol(No_warp_vec(:,1), No_warp_vec(:,2));
      dt_warp = mod(to_warp - dt, 2*pi);

      [a,b] = pol2cart(dt_warp, 1);
      vec_warp = [a,b];

      point_warp = Nr_warp;
      vec_warp = vec_warp ./ sqrt(sum(vec_warp.^2));

      N_warp = nodes_tree_warp(root_tree_node_idx,:) + vec_warp;

      N_warp_vec = N_warp - Nr_warp;
      [t_warp] = cart2pol(N_warp_vec(:,1), N_warp_vec(:,2));

      if abs(mod(t_warp - to_warp, 2*pi) - mod(t - to, 2*pi)) > 10^-5

        v = vec_warp;
        l = No_warp_vec;
        vec_warp = 2*((v*l') ./ (l*l')) * l - v;

        point_warp = Nr_warp;
        vec_warp = vec_warp ./ sqrt(sum(vec_warp.^2));

        N_warp = nodes_tree_warp(root_tree_node_idx,:) + vec_warp;

        N_warp_vec = N_warp - Nr_warp;
        [t_warp] = cart2pol(N_warp_vec(:,1), N_warp_vec(:,2));

        assert(abs(mod(t_warp - to_warp, 2*pi) - mod(t - to, 2*pi)) <= 10^-5)

      end

      control_points{ni} = point_warp;
      control_vecs{ni} = vec_warp;

    else

      Nr = nodes_tree(root_tree_node_idx,:);
      No = nodes_tree(other_tree_node_idx,:);

      N_vec = N - Nr;
      No_vec = No - repmat(Nr, size(No,1), 1);

      [t] = cart2pol(N_vec(:,1), N_vec(:,2));
      [to] = cart2pol(No_vec(:,1), No_vec(:,2));

      bounds = [-inf, inf];

      for i = -1:1
        d = t - (to + i*2*pi);
        if any(d > 0)
          bounds(2) = min(bounds(2), min(d(d>0)));
        end

        d = t - (to + i*2*pi);
        if any(d < 0)
          bounds(1) = max(bounds(1), max(d(d<0)));
        end
      end

      dt = thetaDist(t, to);


      below = find(abs(abs(bounds(1)) - dt) < 10^-5, 1, 'first');
      if isempty(below)
        below = find(abs(abs(bounds(1)) - (2*pi - dt)) < 10^-5, 1, 'first');
      end

      above = find(abs(abs(bounds(2)) - dt) < 10^-5, 1, 'first');
      if isempty(above)
        above = find(abs(abs(bounds(2)) - (2*pi - dt)) < 10^-5, 1, 'first');
      end

      assert(~isempty(below))
      assert(~isempty(above))

      dt_below = mod(t - to(below), 2*pi) - 2*pi;
      dt_above = mod(t - to(above), 2*pi);

      %       [dt_below, dt_above]
      fraction_below = abs(dt_below) ./ (abs(dt_below) + abs(dt_above));
      fraction_above = 1-fraction_below;


      Nr_warp = nodes_tree_warp(root_tree_node_idx,:);
      No_warp = nodes_tree_warp(other_tree_node_idx,:);

      %       N_vec_warp = N_warp - Nr_warp;
      No_vec = No_warp - repmat(Nr_warp, size(No,1), 1);

      [to_warp] = mod(cart2pol(No_vec(:,1), No_vec(:,2)), 2*pi);

      fraction_above = 0.5;

      if (to_warp(below) < to_warp(above))

        new_angle = mod(to_warp(below) + (to_warp(above)-to_warp(below))*fraction_above, 2*pi);

      else

        to_warp_shift = mod(to_warp - pi, 2*pi);
        assert(to_warp_shift(below) < to_warp_shift(above))
        new_angle = mod(to_warp_shift(below) + (to_warp_shift(above)-to_warp_shift(below))*fraction_above + pi, 2*pi);

      end

      [a,b] = pol2cart(new_angle - pi, 1);
      vec = [a,b];
      point = Nr_warp;

      control_points{ni} = point;
      control_vecs{ni} = vec;

    end

  end

end


control_points = cat(1,control_points{:});
control_vecs = cat(1,control_vecs{:});
