
clear all;
rand('twister',5489)

path(path, '/Applications/MATLAB_R2008a/toolbox/cvx/')
path(path, './ann_mwrapper')


DATASET = 'bart';
% DATASET = 'bike';

SMOOTH_LAMBDA = 200;
SIMILAR_LAMBDA = 50;
ZIPPER_LAMBDA = 10;

load bart_graph
start_node_idx = 4;
nodes = graph.nodes;
im = double(imread('Data/Map.png'))/255;

graph.costs = graph.costs + rand(size(graph.costs))/1000000;

[costs, ancestries] = MinTraverse(graph, start_node_idx,  'costs');
costs = costs ./ max(costs);

assert(length(unique(costs)) == length(costs));

[sorted, traversal] = sort(costs, 'ascend');
reverse_traveral = flipud(traversal);
traversal = traversal';
reverse_traveral = reverse_traveral';



N = size(nodes,1);

% node_center = nodes(start_node_idx,:);
% nodes = nodes - repmat(node_center, size(nodes,1), 1);

parents = {};
ancestry_mat = {};
for i = 1:length(ancestries)
  if length(ancestries{i}) > 1
    parents{i} = ancestries{i}(end-1);
  else
    parents{i} = 0;
  end
  ancestry_mat{i} = sparse(ancestries{i}', ones(size(ancestries{i}))', ones(size(ancestries{i}))', N, 1);
end
parents = cat(1,parents{:});
ancestry_mat = cat(2,ancestry_mat{:})';

children = {};
for i = 1:N
  children{i} = find(parents == i)';
  if isempty(children{i})
    children{i} = [];
  end
end

offsets = {};
for i = 1:length(parents)
  if parents(i) == 0
    offsets{i} = nodes(start_node_idx,:);
  else
    offsets{i} = nodes(i,:) - nodes(parents(i),:);
  end
end

offsets = cat(1,offsets{:});
offsets_init = offsets;

edges = {};
for i = 1:length(ancestries)
  edges{i} = [ancestries{i}(1:end-1)', ancestries{i}(2:end)'];
end
edges = cat(1,edges{:});
edges = unique(edges, 'rows');

edge_mat = sparse(edges(:,1), edges(:,2), ones(size(edges,1),1), N, N);
edge_mat = double(edge_mat | edge_mat');

[i,j] = find(triu(edge_mat));
edge_idx = [i,j];
flip = costs(edge_idx(:,1)) > costs(edge_idx(:,2));
edge_idx(flip,:) = edge_idx(flip,[2,1]);


subtree_branching = nan(1,N);
for i = reverse_traveral
  if isempty(children{i})
    subtree_branching(i) = 1;
  else
    subtree_branching(i) = sum(subtree_branching(children{i}));
  end
end

subtree_n = nan(1,N);
for i = reverse_traveral
  if isempty(children{i})
    subtree_n(i) = 1;
  else
    subtree_n(i) = 1+sum(subtree_n(children{i}));
  end
end


subtree_lengths = nan(N,1);
for i = reverse_traveral
  if isempty(children{i})
    subtree_lengths(i) = costs(i) - costs(parents(i));
  elseif parents(i) == 0
    continue
  else
    subtree_lengths(i) = costs(i) - costs(parents(i)) + max(subtree_lengths(children{i}));
  end
end

% subtree_amounts = subtree_n ./ subtree_lengths';
% subtree_amounts = (subtree_n) + subtree_lengths';
% subtree_amounts = subtree_n;
subtree_amounts = subtree_n;
% subtree_amounts = subtree_branching;

node_depths = nan(N,1);
i = start_node_idx;

for d = 0:inf
  node_depths(i) = d;
  i = cat(2,children{i});
  if isempty(i)
    break
  end
end



global ordered_children
ordered_children = {};
orderChildren(nodes, children, start_node_idx, [], start_node_idx);
children = ordered_children;


global nodes_warp nodes_fencepost nodes_triangles
nodes_warp = nan(N,2);
nodes_warp(start_node_idx,:) = [0, 0];
nodes_fencepost = {};
nodes_triangles = cell(1,N);
data = [];
data.nodes = nodes;
data.costs = costs;
data.children = children;
data.subtree_amounts = subtree_amounts;
data.subtree_lengths = subtree_lengths;
data.subtree_n = subtree_n;
data.start_node_idx = start_node_idx;
data.edge_idx = edge_idx;
data.traversal = traversal;
data.node_depths = node_depths;
triLay(data, [], start_node_idx);

% figure;
% renderTree(nodes_warp, start_node_idx, costs, edge_idx, 1);
% print('-dpng', 'vis4.png');

[theta, rho] = cart2pol(nodes_warp(:,1), nodes_warp(:,2));
theta = mod(theta, 2*pi);
fprintf('error: %e\n', sum((rho - costs).^2))

figure;
patches = cat(3,nodes_triangles{traversal});
patch(squeeze(patches(:,1,:)), squeeze(patches(:,2,:)), rand(1,size(patches,3)), 'EdgeColor', 'none', 'FaceAlpha', .25)
cmap = colormap('hsv');
renderTree(nodes_warp, start_node_idx, [], edge_idx, 1);
% print('-depsc', ['tree1_', DATASET, '.eps']);

figure;
[nodes_tree, nodes_mesh, tris] = tesselate(nodes, edges);
% print('-depsc', ['tree2_', DATASET, '.eps']);

edge_children = {};
edge_children_mults = {};
for ei = 1:size(edges,1)
  edge = edges(ei,:);
  vec = nodes(edge(2),:) - nodes(edge(1),:);
  p1 = nodes(edge(1),:);

  diff = nodes_tree - repmat(p1, size(nodes_tree,1), 1);

  on_line = find(abs((diff(:,2) * -vec(1) + diff(:,1) * vec(2))) <= 10^-8);

  i = find(vec ~= 0, 1, 'first');
  mult = diff(on_line,i) ./ vec(i);

  in_line = find((mult >= -10^-8) & (mult <= 1+10^-8));
  [sorted, sortidx] = sort(mult(in_line), 'ascend');

  idx = on_line(in_line(sortidx));

  edge_children{ei} = idx;
  mult = mult(in_line);
  edge_children_mults{ei} = mult(sortidx);
end

nodes_tree_warp = nan(size(nodes_tree));
[tf, loc] = ismember(nodes_tree, nodes, 'rows');
nodes_tree_warp(tf,:) = nodes_warp(loc(tf),:);

for ei = 1:size(edges,1)
  idx = edge_children{ei};

  p1 = nodes_warp(edges(ei,1),:);
  p2 = nodes_warp(edges(ei,2),:);
  vec = p2 - p1;

  mult = edge_children_mults{ei};

  nodes_tree_warp(idx,:) = repmat(vec, length(mult), 1) .* repmat(mult, 1, 2) + repmat(p1, length(mult), 1);

end


D = distMat(nodes_mesh, nodes_mesh);
nodes_mesh_match = sparse((D == 0));


figure;
line([nodes(edges(:,1),1), nodes(edges(:,2),1)]', [nodes(edges(:,1),2), nodes(edges(:,2),2)]', 'Color', 'k');
hold on;
scatter(nodes(:,1), nodes(:,2), 20, 'ko', 'MarkerFaceColor', [0,0,0])
axis([0,1,0,1])

nodes_all = [nodes_tree; nodes_mesh];
patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', repmat(rand(size(tris(:,3))), 1, 3)', 'EdgeColor', 'none', 'FaceAlpha', .25)
axis image off ij;
drawnow
% print('-depsc', ['tree3a_', DATASET, '.eps']);


figure;
patches = cat(3,nodes_triangles{traversal});
patch(squeeze(patches(:,1,:)), squeeze(patches(:,2,:)), rand(1,size(patches,3)), 'EdgeColor', 'none', 'FaceAlpha', .25)
cmap = colormap('hsv');
renderTree(nodes_warp, start_node_idx, [], edge_idx, 1);
scatter(nodes_tree_warp(:,1), nodes_tree_warp(:,2), 20, 'ko', 'filled')

% print('-depsc', ['tree3b_', DATASET, '.eps']);

edges_tree = [tris(:,1), tris(:,2); tris(:,2), tris(:,3); tris(:,1), tris(:,3);];
edges_tree = edges_tree(all(edges_tree <= size(nodes_tree,1),2),:);
edges_tree = sort(edges_tree, 2, 'ascend');
edges_tree = unique(edges_tree, 'rows');


% Find the control point and control vectors for each mesh point.
[control_points, control_vecs] = findControlPoints(nodes_tree, nodes_tree_warp, edges_tree, nodes_mesh, tris);


mults = [0.02, 0.05, 0.2];
for mult_i = 1:length(mults)

  mult = mults(mult_i);
  nodes_mesh_warp = control_points + mult * control_vecs;
  nodes_all = [nodes_tree_warp; nodes_mesh_warp];

  figure;
  clf
  renderTree(nodes_warp, start_node_idx, [], edge_idx, 1); ax = axis;
  patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', repmat(rand(size(tris(:,3))), 1, 3)', 'EdgeColor', 'none', 'FaceAlpha', .25); axis image off; plotHelp_setsize(1,1,.02)
  axis(ax);
  drawnow;
  
  mult = mults(mult_i);
%   print('-depsc', ['tree4_', num2str(mult_i) ,'_', DATASET, '.eps']);
end


nodes_interp = interpolate_tree(nodes_tree_warp, edges_tree, .005);
nodes_interp_all = cat(1, nodes_interp{:});

nodes_interp_parents = {};
for i = 1:length(nodes_interp)
  nodes_interp_parents{i} = repmat(i, length(nodes_interp{i}), 1);
end
nodes_interp_parents = cat(1,nodes_interp_parents{:});

tmp = unique(nodes_interp_all, 'rows');
[vx, vy] = voronoi(tmp(:,1), tmp(:,2));

p1 = [vx(1,:)', vy(1,:)'];
p2 = [vx(2,:)', vy(2,:)'];
V = unique([p1; p2], 'rows');

[tf, loc1] = ismember(p1, V, 'rows');
[tf, loc2] = ismember(p2, V, 'rows');
E = [loc1, loc2];

I = LineSegmentIntersect(V, nodes_tree_warp, E, edges_tree);
E = E(setdiff(1:size(E,1), I(:,1)),:);

figure;
renderTree(nodes_warp, start_node_idx, [], edge_idx, 1);
hold on;
line([V(E(:,1),1), V(E(:,2),1)]', [V(E(:,1),2), V(E(:,2),2)]', 'Color', 'g', 'LineWidth', 3)

% print('-depsc', ['tree5_', DATASET, '.eps']);


control_maxes = nan(size(control_points,1),1);
for ci = 1:size(control_points,1)

  p1 = control_points(ci,:);
  v1 = control_vecs(ci,:);

  ps = [p1 + v1*(10^-8); p1 + v1*(10^8)];

  I = LineSegmentIntersect(ps, V, [1,2], E);
  I = I(:,2);

  max_d = inf;
  for j = 1:size(I,1)
    p2 = V(E(I(j),1),:);
    v2 = V(E(I(j),2),:) - V(E(I(j),1),:);

    %     clf;
    %     line([p1(1) + [0; v1(1)]], [p1(2) + [0; v1(2)]]); hold on;
    %     line([p2(1) + [0; v2(1)]], [p2(2) + [0; v2(2)]])
    %     drawnow;

    %     p1(1) + d1*v1(1) == p2(1) + d2*v2(1)
    %     p1(2) + d1*v1(2) == p2(2) + d2*v2(2)
    %     d1 > 0
    %     0 <= d2 <= 1

    d1 = -((-p1(2)*v2(1) + p2(2)*v2(1) + p1(1)*v2(2) - p2(1)*v2(2))/(-v1(2)*v2(1) + v1(1)*v2(2)));
    d2 = -((-p1(2)*v1(1) + p2(2)*v1(1) + p1(1)*v1(2) - p2(1)*v1(2))/(-v1(2)*v2(1) + v1(1)*v2(2)));

    if (d1 < -10^-10) || ~( (-10^-10 <= d2) && (d2 <= 1+10^-10))
      continue
    end

    max_d = min(max_d, d1);
  end

  control_maxes(ci) = max_d;

end


edges_mesh = [tris(:,[1,2]); tris(:,[2,3]); tris(:,[3,1])];
edges_mesh = edges_mesh(all(edges_mesh > size(nodes_tree,1),2),:) - size(nodes_tree,1);
edges_mesh = unique(sort(edges_mesh, 2, 'ascend'), 'rows');
E = edges_mesh;

nodes_mesh_warp = control_points + repmat(control_maxes, 1, 2) .* control_vecs;
nodes_all = [nodes_tree_warp; nodes_mesh_warp];

tri_color = repmat(rand(size(tris(:,3))), 1, 3)';

figure;
clf
renderTree(nodes_warp, start_node_idx, [], edge_idx, 1);
patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', tri_color, 'EdgeColor', 'none', 'FaceAlpha', .25); axis image off; plotHelp_setsize(1,1,.02)
drawnow;
% print('-depsc', ['tree6a_', DATASET, '.eps']);

control_maxes = min(.25, control_maxes);



M = control_maxes;
P = control_points;
V = control_vecs;

V(isnan(V)) = 0;


tree_interp = interpolate_tree(nodes_tree, edges_tree, .001);
tree_interp = cat(1,tree_interp{:});
D = distMat(nodes_mesh, tree_interp);
nodes_mesh_distance = sqrt(min(D, [], 2));


cvx_begin
variable X(size(V,1), 1)
loss = SMOOTH_LAMBDA*sum((X(E(:,1)) - X(E(:,2))).^2) - sum(X);

loss = loss + SIMILAR_LAMBDA * sum((nodes_mesh_distance - X).^2);

minimize(loss)
subject to
X >= 0
X <= M
cvx_end

nodes_mesh_warp = P + repmat(X, 1, 2) .* V;


nodes_all = [nodes_tree_warp; nodes_mesh_warp];

figure;
clf
renderTree(nodes_warp, start_node_idx, [], edge_idx, 1);
ax = axis;
patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', tri_color, 'EdgeColor', 'none', 'FaceAlpha', .25); axis image off; plotHelp_setsize(1,1,.02)
axis(ax);
drawnow;
% print('-depsc', ['tree6b_', DATASET, '.eps']);


D = distMat(nodes_mesh, nodes_mesh);
[i,j] = find(triu(D < 10^-12,1));
nodes_mesh_pairings = [i,j];

nodes_mesh_unique = unique(nodes_mesh, 'rows');
edges_mesh_unique = {};
for ei = 1:size(edges_mesh,1)
  n1 = nodes_mesh(edges_mesh(ei,1),:);
  n2 = nodes_mesh(edges_mesh(ei,2),:);
  e1 = find((nodes_mesh_unique(:,1) == n1(1)) & (nodes_mesh_unique(:,2) == n1(2)));
  e2 = find((nodes_mesh_unique(:,1) == n2(1)) & (nodes_mesh_unique(:,2) == n2(2)));
  edges_mesh_unique{ei} = [e1, e2];
end
edges_mesh_unique = sort(unique(cat(1,edges_mesh_unique{:}), 'rows'), 2, 'ascend');
edges_mesh_unique = [edges_mesh_unique; edges_mesh_unique(:,[2,1])];

edges_mesh_mat = sparse(edges_mesh(:,1), edges_mesh(:,2), ones(size(edges_mesh(:,1))), size(nodes_mesh,1), size(nodes_mesh,1));
edges_mesh_mat = edges_mesh_mat | edges_mesh_mat';

edges_mesh_unique_mat = sparse(edges_mesh_unique(:,1), edges_mesh_unique(:,2), ones(size(edges_mesh_unique(:,1))), size(nodes_mesh_unique,1), size(nodes_mesh_unique,1));
edges_mesh_unique_mat = edges_mesh_unique_mat | edges_mesh_unique_mat';

nodes_mesh_unique_mapping = sparse(distMat(nodes_mesh_unique, nodes_mesh) == 0)';

% find all edges whos true degree is one more than the degree of covered
% edges. Keep running history of each node. The history of a new node is
% the union of the history of its predecessors.

history = sparse(size(nodes_mesh_unique,1), size(nodes_mesh_unique,1));
ordering = {};

node_visited = false(size(history,1),1);

last_count = inf;

for iter = 1:inf
  fprintf('%d ', last_count)

  for f = 1:length(node_visited)

    if sum(edges_mesh_unique_mat(:,f) & ~node_visited) == 1
      h = any(history(:, edges_mesh_unique_mat(:,f) & node_visited),2);
      h(f) = true;
      history(:,f) = h;
      node_visited(f) = true;
      ordering{length(ordering)+1} = f;
    end

  end

  count = sum(node_visited);

  if count == last_count
    break
  end

  last_count = count;

end
fprintf('\n')

ordering_all = cat(1,ordering{:})';

for i = 1:length(ordering_all)
  if i > length(ordering_all)
    break
  end
  idx = 1:length(ordering_all);
  ordering_all = ordering_all((idx <= i) | (ordering_all ~= ordering_all(i)));
end



% Find the connected components of the unique mesh edges, excluding the
% border edges.

E = edges_mesh_unique_mat;

D = distMat(nodes_mesh_unique, nodes_tree);
D = D - repmat(min(D,[],2), 1, size(D,2));
border_node = find(sum(D < 10^-12,2) < 2);

% border = 0.05;
% border_node = find((nodes_mesh_unique(:,1) < border) | (nodes_mesh_unique(:,2) < border) | (nodes_mesh_unique(:,1) > (1-border)) | (nodes_mesh_unique(:,2) > (1-border)));

E(border_node,:) = false;
E(:,border_node) = false;
[S,C] = graphconncomp(E, 'directed', false);
C(border_node) = 0;
[N,X] = hist(C, min(C):max(C));
C(ismember(C, X(N == 1))) = 0;
C = C';
[u, ui, uj] = unique(C);
C = uj - 1;
[tf, C] = ismember(C, randperm(max(C)));
C(C==0) = nan;

fprintf('%d zippers\n', length(unique(C(~isnan(C)))))

% Make separate orders for each connected component in the history.
orderings = {};
for c = min(C):max(C)
  idx = find(C == c);
  orderings{c} = ordering_all(ismember(ordering_all, idx));
end

figure;
V = nodes_mesh_unique;
E = edges_mesh_unique;
nodes_all = [nodes_tree; nodes_mesh];
patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', tri_color, 'EdgeColor', 'none', 'FaceAlpha', .25); hold on;
line([V(E(:,1),1), V(E(:,2),1)]', [V(E(:,1),2), V(E(:,2),2)]', 'Color', 'k')
colors = 'rgbcmykrgbcmykrgbcmykrgbcmykrgbcmykrgbcmykrgbcmykrgbcmykrgbcmyk';
for c = min(C):max(C)  
  scatter(V(orderings{c},1), V(orderings{c},2), 10+(60/(length(orderings{c})-1))*[0:(length(orderings{c})-1)], colors(c), 'filled'); hold on;
end

axis image equal ij off;
plotHelp_setsize(1,1,0.02)
% print('-depsc', ['tree7_', DATASET, '.eps']);


nodes_mesh_warp_init = nodes_mesh_warp;

% for ZIPPER_LAMBDA = [0, 10.^[-5, -2, 0, 1, 2, 3]]
nodes_mesh_warp = nodes_mesh_warp_init;

for oi = 1:length(orderings)
  ordering = orderings{oi};

  % Loop through according to the order, merging as you go. Record the loss
  % at each point. Then, find the point where the loss was largest, and take
  % that set of mergings, and leave the rest as is.
  losses = nan(size(ordering));
  nodes_merge = nan(size(nodes_mesh_warp));

  for order_i = 1:length(ordering)

    unique_node_i = ordering(order_i);
    repeat_node_i = find(nodes_mesh_unique_mapping(:,unique_node_i));

    m = any(history(:,unique_node_i),2);

    %     line([V(E(:,1),1), V(E(:,2),1)]', [V(E(:,1),2), V(E(:,2),2)]', 'Color', 'k'); hold on;
    %     scatter(V(m,1), V(m,2), 'bo', 'filled')
    %     scatter(V(unique_node_i,1), V(unique_node_i,2), 'ro', 'filled')


    m(unique_node_i) = 0;
    ancestry = find(m);
%     assert(all(ismember(ancestry, ordering(1:(order_i-1)))))

    if isempty(ancestry)

      losses(order_i) = 0;
      nodes_merge(repeat_node_i,:) = nodes_mesh_warp(repeat_node_i,:);

    else

      neighbors = find(any(edges_mesh_mat(:,repeat_node_i),2));
      predecessor = neighbors(all(~isnan(nodes_merge(neighbors,:)),2),:);
      followers = neighbors(~all(~isnan(nodes_merge(neighbors,:)),2),:);
      self = repeat_node_i;

      nodes_self = nodes_mesh_warp(self,:);
      nodes_self_mean = mean(nodes_self,1);

      %       if ~(all(all(distMat(nodes_merge(predecessor,:), nodes_merge(predecessor,:)) < 10^-12)))
      nodes_pred = nodes_merge(predecessor,:);
      [u, ui, uj] = unique(nodes_pred, 'rows');

      pred_adjacent = {};
      for i = 1:size(u,1)
        belong = find(all(nodes_pred == repmat(u(i,:), size(nodes_pred,1), 1),2));
        pred_adjacent{i} = find(any(edges_mesh_mat(:,predecessor(belong)),2));
        pred_adjacent{i} = intersect(self, pred_adjacent{i});
      end
      nodes_pred = u;

      %         keyboard
      %         break
      %       else
      %         nodes_pred = nodes_merge(predecessor(1),:);
      %       end

      nodes_foll = nodes_mesh_warp(followers,:);

%       L0 = 0;
%       for i = 1:length(pred_adjacent)
%         L0 = L0 + sum(sqrt(distMat(nodes_pred(i,:), nodes_mesh_warp(pred_adjacent{i},:))));
%       end
%       %       L0 = sum(sqrt(distMat(nodes_pred, nodes_self)));
% 
%       [ei,ej] = find(edges_mesh_mat(self, followers));
%       for i = 1:length(ei)
%         L0 = L0 + sqrt(distMat( nodes_self(ei(i),:), nodes_foll(ej(i),:)));
%       end
% 
%       L1 = sum(sqrt(distMat(nodes_pred, nodes_self_mean)));
%       for i = 1:length(ei)
%         L1 = L1 + sqrt(distMat( nodes_self_mean, nodes_foll(ej(i),:)));
%       end
%       L1 = L1 + ZIPPER_LAMBDA*sum(sqrt(distMat(nodes_self, nodes_self_mean)));

      L0 = sum(cellfun(@(x) size(x, 1), pred_adjacent)); % Number of edges formed by preceeding nodes with current nodes
      L0 = L0 + size(nodes_self,1); % Number of edges formed by current nodes with following nodes
      
      L1 = size(nodes_pred,1); % Number of edges formed by preceeding nodes with merged node
      L1 = L1 + size(nodes_foll,1); % Number of edges formed by merged node with following nodes
      L1 = L1 + ZIPPER_LAMBDA*sum(sqrt(distMat(nodes_self, nodes_self_mean)));

      losses(order_i) = L1 - L0;

      nodes_merge(self,:) = repmat(nodes_self_mean, length(self), 1);

    end

  end
  losses = cumsum(losses);
  best = find(losses == min(losses), 1, 'last');

  repeat_node_i = find(any(nodes_mesh_unique_mapping(:,ordering(1:best)),2));
  nodes_mesh_warp(repeat_node_i,:) = nodes_merge(repeat_node_i,:);

end


nodes_all = [nodes_tree; nodes_mesh];
nodes_all_warp = [nodes_tree_warp; nodes_mesh_warp];


figure;
clf
renderTree(nodes_warp, start_node_idx, [], edge_idx, 1); ax = axis;
patch([nodes_all_warp(tris(:,1),1), nodes_all_warp(tris(:,2),1), nodes_all_warp(tris(:,3),1)]', [nodes_all_warp(tris(:,1),2), nodes_all_warp(tris(:,2),2), nodes_all_warp(tris(:,3),2)]', tri_color, 'EdgeColor', 'none', 'FaceAlpha', .25); axis image off; plotHelp_setsize(1,1,.02)
axis(ax);
plotHelp_setsize(1,1,.02)
drawnow;
% end

% print('-depsc', ['tree8_', DATASET, '.eps']);

% return

fprintf('Warping the image. This will take a while.\n');
im_warp = warpImage(im, nodes_all, nodes_all_warp, tris, 2048);

graph.label_idx = [  63   115   106    15    28];

figure;

nodes_all = [nodes_tree; nodes_mesh];
image([min(nodes_all(:,1)), max(nodes_all(:,1))],[min(nodes_all(:,2)), max(nodes_all(:,2))],im)
renderTree(nodes, start_node_idx, costs, edge_idx, 1);
% patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', repmat(rand(size(tris(:,3))), 1, 3)', 'EdgeColor', 'none', 'FaceAlpha', 0); axis image off; plotHelp_setsize(1,1,.02)
set(gca, 'XLim', [min(nodes_all(:,1)), max(nodes_all(:,1))])
set(gca, 'YLim', [min(nodes_all(:,2)), max(nodes_all(:,2))])

if strcmp(DATASET, 'bart')
  % label_points = {};
  for ii = 1:length(graph.label_idx)
    i = graph.label_idx(ii);
    text(nodes_all(i,1)+.015, nodes_all(i,2), graph.label_name{ii}, 'FontSize', 12, 'FontWeight', 'bold', 'Color', 'r')
    %   [x,y] = ginput(1);
    %   label_points{ii} = [x,y];
  end
  % label_points = cat(1,label_points{:});
  % D = distMat(label_points, nodes_all);
  % [i,j] = find((D == repmat(min(D, [], 2), 1, size(D,2)))')
end

drawnow;

% print('-depsc', ['tree0_', DATASET, '.eps']);


figure

nodes_all = [nodes_tree_warp; nodes_mesh_warp];
image([min(nodes_all(:,1)), max(nodes_all(:,1))],[min(nodes_all(:,2)), max(nodes_all(:,2))],im_warp)
renderTree(nodes_warp, start_node_idx, [], edge_idx, 1);
% patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', repmat(rand(size(tris(:,3))), 1, 3)', 'EdgeColor', 'none', 'FaceAlpha', 0); axis image off; plotHelp_setsize(1,1,.02)
set(gca, 'XLim', [min(nodes_all(:,1)), max(nodes_all(:,1))])
set(gca, 'YLim', [min(nodes_all(:,2)), max(nodes_all(:,2))])

if strcmp(DATASET, 'bart')
  y_offsets = [-0.02, 0.02, 0, 0, 0];
  for ii = 1:length(graph.label_idx)
    i = graph.label_idx(ii);
    text(nodes_all(i,1)+.015, y_offsets(ii)+nodes_all(i,2), graph.label_name{ii}, 'FontSize', 12, 'FontWeight', 'bold', 'Color', 'r')
  end
end
  
drawnow;
% print('-depsc', ['tree9_', DATASET, '.eps']);

