function accelerations = initAccelerations(path, biker)

  CONSTANTS

  s_init = biker.stop_speed;  
  K1 = biker.K1;
  K2 = biker.K2;
  M = biker.mass;

  bt = biker.timing;
  be = biker.effort;
  bir = biker.init_regularity;
  bp = biker.effort_power;
  
  d = path.true_dist;
  g = path.grade;
  len = path.n_samples;
    
  max_e_rate = 1;
  
  cvx_quiet(true)
%   cvx_quiet(false)
  
  cvx_solver sdpt3
  
  cvx_precision low
  
  tic
  fprintf('Initializing acceleration... ')
  
  cvx_begin

    variable s(len+1)
    
    minimize( sum((bt*d .* inv_pos((s(2:end)+s(1:end-1))/2)) + be*max(0, (d.*(K1 + K3*M*g + K2*pow_p((s(2:end)+s(1:end-1))/2,2)))/1000)) + bir*sum((s(2:end)-s(1:end-1)).^2) )
    
    subject to
      s >= biker.min_speed;
      s <= biker.max_speed;
      s(1) == s_init;
      s(end) == s_init;
      
  cvx_end
  
  fprintf('done in %0.2f seconds.\n', toc);
  
  s0 = s(1:end-1);
  s1 = s(2:end);
  accelerations = (s1.^2 - s0.^2)./(2*d);
  
  accelerations = imfilter(accelerations, fspecial('Gaussian', 5*biker.init_smoothness,biker.init_smoothness), 'replicate', 'same');
  

end



    
    % % This is trying to minimize the actual piecewise cost.
%     
%     sqrt_a = sqrt(a);
%     sqrt_d = sqrt(d);
%     sqrt_ad_sum = sqrt(ad_sum);
%     
%     cvx_begin
%       variable sp_norm
%       variable a
%     
% %       minimize( ((-s0 + sqrt(2)*sqrt(a*d + ad_sum))*(2000*bt + be*(s0 + sqrt(2)*sqrt(a*d + ad_sum))*(K1 + K3*(g + (a*K4)/GRAVITY)*M + 1/4*K2*(s0 + sqrt(2)*sqrt(a*d + ad_sum))^2)))/(2000*a) )
% 
% %       minimize( ((-s0 + sqrt(2)*sqrt(a*d + ad_sum)) ...
% %         *(2000*bt + be*(s0 + sqrt(2)*sqrt(a*d + ad_sum)) ...
% %           *(K1 + K3*(g + (a*K4)/GRAVITY)*M ...
% %             + 1/4*K2*(s0 + sqrt(2)*sqrt(a*d + ad_sum))^2)))/(2000*a) )
% 
% %       minimize(sqrt(2)*sqrt(a*d + ad_sum))
% 
% %       minimize(sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))
% 
% %       minimize( ((-s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))*(2000*bt + be*(s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))*(K1 + K3*(g + ((sqrt_a^2)*K4)/GRAVITY)*M + 1/4*K2* (s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))^2 )))/(2000*(sqrt_a^2)) )
% %         minimize( ((-s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))*(2000*bt + be*(s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))*(K1 + K3*(g + ((sqrt_a^2)*K4)/GRAVITY)*M + 1/4*K2* ...
% %           square_pos( s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]) ) ...
% %           )))/(2000*(sqrt_a^2)) )
% %       minimize(sqrt_a^2)
% 
% %     minimize( ((-s0 + sqrt(2)*sp_norm)*(2000*bt + be*(s0 + sqrt(2)*sp_norm)*(K1 + K3*(g + ((sqrt_a^2)*K4)/GRAVITY)*M + 1/4*K2* (s0 + sqrt(2)*sp_norm)^2 )))/(2000*(sqrt_a^2)) )
% 
% 
% %     minimize( ((-s0 + sqrt(2)*sqrt(a*d + ad_sum))*(2000*bt + be*(s0 + sqrt(2)*sqrt(a*d + ad_sum))*(K1 + K3*(g + (a*K4)/GRAVITY)*M + 1/4*K2*(s0 + sqrt(2)*sqrt(a*d + ad_sum))^2)))/(2000*a) )
%     minimize(sp_norm)
%     
%       subject to 
%         sp_norm >= (a*d + ad_sum)
%         
%     cvx_end
%     
%     keyboard

% function accelerations = initAccelerations(path, biker)
% 
%   CONSTANTS
% 
%   s_init = biker.stop_speed;  
%   K1 = biker.K1;
%   K2 = biker.K2;
%   M = biker.mass;
%   energy_power = biker.energy_power;
%   
%   len = path.n_samples;
%   
%   % This is scrap
%   accelerations = ACCELERATIONS_INIT*ones(len,1);
%   
%   distances = path.true_dist;
%   grades = path.grade;
%     
%   speeds = sqrt(2)*sqrt(0.5*s_init^2 + cumsum(accelerations .* distances));  
%   speeds_before = [s_init; speeds(1:end-1)];
%   times = (speeds - speeds_before)./accelerations;
%       
%   % Take the midpoint speed
%   sm = (speeds + speeds_before)/2;
%   energies = times .* (sm .* (K1 + (K2 .* (sm.^2)) + (K3*M.*(grades + K4/GRAVITY .* accelerations))))/1000;
%   energies(energies < 0) = 0;
% 
%   total_time = sum(times);
%   total_energy = sum(1/energy_power * energies.^energy_power);
%   
%   cost = ...
%       biker.timing*total_time ...
%     + biker.effort*total_energy;
%   
%  
%   % This is for realsies
% %   accelerations = ACCELERATIONS_INIT*ones(len,1);
%   
%   speed = s_init;
%   ad_sum = 0.5*s_init^2;
%   
%   step_energy = 200/len;
%   
%   for step = 1:len
%     speed_before = speed;
%     
%     % Shorthand
%     d = distances(step);
%     s0 = speed_before;
%     bt = biker.timing;
%     be = biker.effort;
%     g = grades(step);
% 
% %     a = accelerations(step);    
% %     s = sqrt(2)*sqrt(ad_sum + a * d);
% %     t = (s - s0)./a;
% 
%     s = 5;
%     a = ((s^2)/2 - ad_sum)/d;
%     t = (s - s0)/a;
%     
%     t_simple = d/(s0+s);
%     
% %     t = (2*d*(s - s0))/(s^2 - 2*ad_sum)
%     e_rate = (K1*s)/1000 + (g*K3*M*s)/1000 + (K2*s^3)/1000 + (K3*K4*M*s^3)/(2000*d*GRAVITY) - (K3*K4*M*s*ad_sum)/(1000*d*GRAVITY);
%     
%     % This is trying to minimize time given an energy constraint
%     max_e_rate = 10;
%     
%     
%     cvx_begin
%       variable s
%     
% %       minimize( (2*d*(s - s0))/(s^2 - 2*ad_sum) )
%       minimize( -(s0+s)/d )
%     
%       subject to 
%         max_e_rate > (K1*s)/1000 + (g*K3*M*s)/1000 + (K2*pow_pos(s,3))/1000 + (K3*K4*M*pow_pos(s,3))/(2000*d*GRAVITY) - (K3*K4*M*s*ad_sum)/(1000*d*GRAVITY)
%         
%     cvx_end
%     
%     s
% 
%     
%     % % This is trying to minimize the actual piecewise cost.
% %     
% %     sqrt_a = sqrt(a);
% %     sqrt_d = sqrt(d);
% %     sqrt_ad_sum = sqrt(ad_sum);
% %     
% %     cvx_begin
% %       variable sp_norm
% %       variable a
% %     
% % %       minimize( ((-s0 + sqrt(2)*sqrt(a*d + ad_sum))*(2000*bt + be*(s0 + sqrt(2)*sqrt(a*d + ad_sum))*(K1 + K3*(g + (a*K4)/GRAVITY)*M + 1/4*K2*(s0 + sqrt(2)*sqrt(a*d + ad_sum))^2)))/(2000*a) )
% % 
% % %       minimize( ((-s0 + sqrt(2)*sqrt(a*d + ad_sum)) ...
% % %         *(2000*bt + be*(s0 + sqrt(2)*sqrt(a*d + ad_sum)) ...
% % %           *(K1 + K3*(g + (a*K4)/GRAVITY)*M ...
% % %             + 1/4*K2*(s0 + sqrt(2)*sqrt(a*d + ad_sum))^2)))/(2000*a) )
% % 
% % %       minimize(sqrt(2)*sqrt(a*d + ad_sum))
% % 
% % %       minimize(sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))
% % 
% % %       minimize( ((-s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))*(2000*bt + be*(s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))*(K1 + K3*(g + ((sqrt_a^2)*K4)/GRAVITY)*M + 1/4*K2* (s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))^2 )))/(2000*(sqrt_a^2)) )
% % %         minimize( ((-s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))*(2000*bt + be*(s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]))*(K1 + K3*(g + ((sqrt_a^2)*K4)/GRAVITY)*M + 1/4*K2* ...
% % %           square_pos( s0 + sqrt(2)*norm([sqrt_a * sqrt_d, sqrt_ad_sum]) ) ...
% % %           )))/(2000*(sqrt_a^2)) )
% % %       minimize(sqrt_a^2)
% % 
% % %     minimize( ((-s0 + sqrt(2)*sp_norm)*(2000*bt + be*(s0 + sqrt(2)*sp_norm)*(K1 + K3*(g + ((sqrt_a^2)*K4)/GRAVITY)*M + 1/4*K2* (s0 + sqrt(2)*sp_norm)^2 )))/(2000*(sqrt_a^2)) )
% % 
% % 
% % %     minimize( ((-s0 + sqrt(2)*sqrt(a*d + ad_sum))*(2000*bt + be*(s0 + sqrt(2)*sqrt(a*d + ad_sum))*(K1 + K3*(g + (a*K4)/GRAVITY)*M + 1/4*K2*(s0 + sqrt(2)*sqrt(a*d + ad_sum))^2)))/(2000*a) )
% %     minimize(sp_norm)
% %     
% %       subject to 
% %         sp_norm >= (a*d + ad_sum)
% %         
% %     cvx_end
% %     
% %     keyboard
% 
%     
%     
%     ad_sum = ad_sum + accelerations(step) * distances(step);
%   end
%   
%   keyboard
% 
% end
% 
% function [accelerations] = initAccelerations(path, biker)
% 
%   CONSTANTS
% 
%   distances = path.true_dist;
%   grades = path.grade;
%   
%   accelerations = ACCELERATIONS_INIT*ones(path.n_samples,1);
%   
%   len = length(accelerations);
% 
%   x = [1:len];
% %   goal_speeds = -(x+1).*(x-len);
% %   goal_speeds = goal_speeds/max(goal_speeds) * (MAX_SPEED-biker.stop_speed) + biker.stop_speed
%   
%   goal_speeds = repmat((biker.min_speed)/2, 1, len);
% %   goal_speeds = repmat((biker.max_speed-biker.stop_speed)/2, 1, len);
% 
% %   goal_speeds = filter2(fspecial('Gaussian', 9, 1), (-23.7 * path.grade + 16.1)*2/3, 'same') + (MAX_SPEED-biker.stop_speed)*1/3;
% 
%   ad_sum = 0;
%   
%   goal_ads = ((goal_speeds./sqrt(2)).^2);
%   for i = 1:len
%     
%     accelerations(i) = (goal_ads(i) - ad_sum)/distances(i);    
%     ad_sum = ad_sum + accelerations(i) * distances(i);
%     
%   end
%   
% end