function path = makePath(lat_range, lon_range, height_map)
  
  CONSTANTS
  
  latlon_dist = sqrt((lat_range(2)-lat_range(1)).^2 + (lon_range(2)-lon_range(1)).^2);
  path.n_samples = ceil(latlon_dist/PATH_INTERVAL);

  lat_step = (lat_range(2)-lat_range(1))/path.n_samples;
  lon_step = (lon_range(2)-lon_range(1))/path.n_samples;

  path.lats = zeros(path.n_samples+1,1);
  path.lons = zeros(path.n_samples+1,1);
  for i = 0:path.n_samples
    path.lats(i+1) = lat_range(1) + lat_step * i;
    path.lons(i+1) = lon_range(1) + lon_step * i;
  end

  [path.us, path.vs] = LatLon2UV(path.lats, path.lons);
  path.heights = interp2(height_map.height, path.us, path.vs);
  path.height_diff = path.heights(2:end) - path.heights(1:end-1);

  [xyz] = LatLon2XYZ(path.lats, path.lons);
  path.arc_dist = sqrt(sum((xyz(2:end,:) - xyz(1:end-1,:)).^2,2));

  path.true_dist = sqrt(path.height_diff.^2 + path.arc_dist.^2); % meters
  path.grade = path.height_diff ./ path.arc_dist; % fraction
  
end