function graph = makeRandomGraph(map, graph, is_simple)

  if nargin < 3
    is_simple = 0;
  end
  rand('twister', graph.seed)
  
  sz = map.sz;
  n_nodes = graph.n_nodes;
  n_edges = graph.n_edges;
  min_dist = graph.min_dist;
  max_degree = graph.max_degree;
  min_angle = graph.min_angle;
    
  % check if any are too close. Do waves of insertions, until we have
  % n_nodes.
  t = cputime;

%   Q = qrandstream('halton',2,'Leap',12,'Skip',1);
  nodes = [];
  last_percent = -100;
  while true
    n_new_nodes = n_nodes - size(nodes,1);
    
    if is_simple == 1
      nx = round(sqrt(n_new_nodes));
      ny = floor(n_new_nodes/nx);
      [x,y] = meshgrid(1:floor(sz(1)/(nx-1)):sz(1), 1:floor(sz(2)/(ny-1)):sz(2));
      x = x(:);
      y = y(:);
      new_nodes = [x,y];
      new_nodes = new_nodes(1:n_new_nodes,:);
    elseif is_simple == 3
      nx = round(sqrt(n_new_nodes));
      ny = floor(n_new_nodes/nx);
      [x,y] = meshgrid(1:floor(sz(1)/(nx-1)):sz(1), 1:floor(sz(2)/(ny-1)):sz(2));
      x = x(:);
      y = y(:);
      new_nodes = [x,y];
      new_nodes = new_nodes(1:n_new_nodes,:);
      new_nodes = new_nodes + (rand(size(new_nodes))-0.5)*7;
      new_nodes = round(min(max(x), max(min(x), new_nodes)));
      
    elseif is_simple == 2
      
%       n_circles = 7;
%       r_diff = 14;

      n_circles = 9;
      r_diff = 10;
      
      center = sz/2;
      
      new_nodes = center;
      ta = prod(sz);
      for c = 1:n_circles
        r = r_diff * c;
        a = pi* ((r + r_diff/2)^2 - (r - r_diff)^2);
        n = floor(a / ta * n_new_nodes);
        for theta = 2*pi*(0:(n-1))/n
          new_nodes = [new_nodes; [r*cos(theta), r*sin(theta)]+center];
        end
      end
      
      n_new_nodes = size(new_nodes,1);
      n_nodes = size(new_nodes,1);
      
    else
%     new_nodes = qrand(Q,n_new_nodes) .* repmat(sz, n_new_nodes,1);
    new_nodes = [rand([n_new_nodes,1])*(sz(1)-2)+1, rand([n_new_nodes,1])*(sz(2)-2)+1];
    end
    
    nodes = [nodes; new_nodes];

%     nodes = [rand([n_nodes,1])*(sz(1)-2)+1, rand([n_nodes,1])*(sz(2)-2)+1];
    D = distMat(nodes, nodes);
    D = triu(D);
    D(D == 0) = Inf;
    [junk, to_remove] = find(D <= min_dist.^2);
    if isempty(to_remove)
      break
    end
    nodes = nodes(setdiff(1:size(nodes,1), to_remove),:);
    percent = round(size(nodes,1)/n_nodes*100);
    if percent ~= last_percent
      fprintf('%d%%, ', percent);
      last_percent = percent;
    end
  end

  fprintf('\nnodes constructed in %0.1f seconds.\n', cputime-t);

  
  t = cputime;
  
  node_dist = triu(distMat(nodes, nodes));
  node_dist(node_dist == 0) = Inf;
  [i,j,d] = find(node_dist);
  [d_sorted, d_sortidx] = sort(d, 'ascend');
  edge_i = i(d_sortidx(~isinf(d_sorted)));
  edge_j = j(d_sortidx(~isinf(d_sorted)));
  
  edges = zeros(n_nodes, n_nodes);

  edge_lines = {};
  edge_lengths = {};
  im_blank = zeros(sz(1), sz(2));
  
  n_edges_added = 0;
  for i = 1:size(edge_i,1)
    e = [edge_i(i), edge_j(i)];
    if (full(sum(edges(e(1),:)>0)) < max_degree) && (full(sum(edges(e(2),:)>0)) < max_degree)
      n_edges_added = n_edges_added + 1;
            
      edge_lines{n_edges_added} = nodes(e,:);
      edge_lengths{n_edges_added} = sqrt(sum((nodes(e(1),:) - nodes(e(2),:)).^2));

      edges(e(1), e(2)) = n_edges_added;
      edges(e(2), e(1)) = n_edges_added;
    end
    
    if n_edges_added > n_edges
      break
    end
  end
  fprintf('edges constructed in %0.1f seconds.\n', cputime-t);

  to_remove = false(length(edge_lines),1);
  t = cputime;
  % find all collisions, and remove the larger line
  if ~is_simple
    for edge_1 = 1:length(edge_lines)
      for edge_2 = (edge_1+1):length(edge_lines)
        if to_remove(edge_1) || to_remove(edge_2)
          continue
        end

        l1 = edge_lines{edge_1};
        p1 = l1(1,:);
        v1 = l1(2,:) - l1(1,:);

        l2 = edge_lines{edge_2};
        p2 = l2(1,:);
        v2 = l2(2,:) - l2(1,:);

        slope1 = v1(2)/v1(1);
        slope2 = v2(2)/v2(1);

        b1 = p1(2) - slope1*p1(1);
        b2 = p2(2) - slope2*p2(1);

        x_int = (b2 - b1)/(slope1-slope2);
        y_int = slope1*x_int + b1; %== slope2*x_int + b2;

        p_int = [x_int, y_int];

        proj1 = (x_int - p1(1)) ./ v1(1);
        proj2 = (x_int - p2(1)) ./ v2(1);

        is_collision = (abs(proj1 - 0.5) <= 0.49) && (abs(proj2 - 0.5) <= 0.49);

        if is_collision
          if edge_lengths{edge_1} >= edge_lengths{edge_2}
            to_remove(edge_1) = true;
            edge_lines{edge_1} = 0*edge_lines{edge_1};
          else
            to_remove(edge_2) = true;
            edge_lines{edge_2} = 0*edge_lines{edge_2};
          end
        end
      end
    end
  end
  to_remove = find(to_remove);
  
  % Should also weed out edges that come from the same node, and have very
  % small angles.
  
  for n = 1:size(nodes,1)
    while true
      cur_edges = edges(n,:);
      cur_edges(ismember(cur_edges, to_remove)) = 0;
      [junk, neighbors, neighbor_edges] = find(cur_edges);
      neighbor_vecs = nodes(neighbors,:) - repmat(nodes(n,:), length(neighbors),1);
      neighbor_vecs = neighbor_vecs ./ repmat(sqrt(sum(neighbor_vecs.^2,2)), 1, 2);

      len = size(neighbor_vecs,1);

      neighbor_dots = repmat(neighbor_vecs(:,1), len, 1) .* reshape(repmat(neighbor_vecs(:,1)', len, 1), [], 1) ...
        + repmat(neighbor_vecs(:,2), len, 1) .* reshape(repmat(neighbor_vecs(:,2)', len, 1), [], 1);
      neighbor_dots = reshape(neighbor_dots, len, len);

      neighbor_angles = triu(real(acos(neighbor_dots))) + tril(Inf*ones(len,len));
      
      if min(neighbor_angles(:)) < min_angle
        [i,j] = find(neighbor_angles == min(neighbor_angles(:)), 1, 'first');
        
        ni = neighbors(i);
        nj = neighbors(j);
        
        di = sum((nodes(n,:) - nodes(ni,:)).^2);
        dj = sum((nodes(n,:) - nodes(nj,:)).^2);
        
        if di > dj
          j = i;
        end
        
        to_remove = [to_remove; neighbor_edges(j)];
        edges(n,edges(n,:) == neighbor_edges(j)) = 0;
        
      else
        break
      end
      
    end
  end
  
  
  edges(ismember(edges, to_remove)) = 0;
  fprintf('collisions removed in %0.1f seconds.\n', cputime-t);
  
  [u_edges] = unique(edges(edges>0));
  u_edge_idx = [1:size(u_edges,1)]';

  for u = u_edges'
    edges(edges == u) = find(u_edges == u);
  end
  edge_lines = edge_lines(u_edges);
  
  graph.nodes = nodes;
  graph.edges = edges>0;
%   graph.lines = edge_lines;
  
  if nargout < 1
    figure(1);
    scatter(nodes(:,1), nodes(:,2)); hold on;
    [edge_i, edge_j] = find(edges);
    for i = 1:size(edge_i,1)
      line( [nodes(edge_i(i),1), nodes(edge_j(i),1)], [nodes(edge_i(i),2), nodes(edge_j(i),2)] );
    end
  end
  
end
