function [cost, part_cost, accelerations, part_stats] = minCost(path, biker, is_start, accelerations)

  if nargin < 4
    accelerations = NaN;
  end
  if nargin < 3
    is_start = 0;
  end

  CONSTANTS

  if isnan(accelerations)

    if PRINT_OUTPUT
      tic;
      fprintf('Initializing accelerations... ');
    end
    
    a = 1;
    for i = 1:10
      accelerations = correctAccelerations(a*ones(path.n_steps,1), biker, path, is_start);
      d = mean(accelerations-a);
      if abs(d) < 0.001;
        break
      end
      s = sign(d);
      a = a*(0.5*(s<=0) + 1.5*(s>0));
    end
    
    if PRINT_OUTPUT
      fprintf('done in %0.2f seconds.\n', toc);
    end
    
  end

  if CHECK_GRAD
    [q,a,n]=checkgrad('pathCost',accelerations, 1e-5, biker, path, 0);
    figure; plot(a,'r-+'); hold on; plot(n,'b'); legend('Analytic','Numerical');
  end

%   [cost, junk, part_cost, part_stats] = pathCost(accelerations, biker, path, is_start);

  if PRINT_OUTPUT
    tic
  end
  
  
  [accelerations, f] = minimize_CG(accelerations,'pathCost', 'correctAccelerations', MINIMIZE_MAX_ITERS, MINIMIZE_STREAK_BOUND, MINIMIZE_CG_STREAK_LENGTH, biker, path, is_start);
  
  fs = zeros(100,1);
  for iter = 1:50
    [accelerations, f] = minimize_CG(accelerations,'pathCost', 'correctAccelerations', 5, Inf, Inf, biker, path, is_start);
    fs(iter) = f(1);
    MINIMIZE_CG_STREAK_LENGTH = 1;
    if iter > MINIMIZE_CG_STREAK_LENGTH
      if all(abs(fs(iter) - fs(iter-MINIMIZE_CG_STREAK_LENGTH:iter)) < MINIMIZE_STREAK_BOUND)
        if PRINT_OUTPUT
          fprintf('Terminated by cost.  ');
        end
        break
      end
    end      

  end
  
%   [accelerations] = minimize_Stoch(accelerations,'pathCost', 'correctAccelerations', MINIMIZE_MAX_ITERS, MINIMIZE_STEP_SIZE, biker, path, is_start);
  if PRINT_OUTPUT
    toc
  end
    
  accelerations = correctAccelerations(accelerations, biker, path, is_start, 1);
  
  if CHECK_GRAD
    [q,a,n]=checkgrad('pathCost',accelerations, 1e-5, biker, path, 0);
    figure; plot(a,'r-+'); hold on; plot(n,'b'); legend('Analytic','Numerical');
  end

  
  [cost, junk, part_cost, part_stats] = pathCost(accelerations, biker, path, is_start);
  
  if RENDER_PLOTS
  
    figure(102);
    clf
    plot(cumsum(path.flat_dists), path.heights(2:end), 'b-'); hold on;
    axis equal
    plot(cumsum(path.flat_dists), sum(path.flat_dists)/4*accelerations/max(abs(accelerations(:))), 'r-')
    plot(cumsum(path.flat_dists), sum(path.flat_dists)/4*part_stats.speeds/max(part_stats.speeds), 'g-')
    plot(cumsum(path.flat_dists), sum(path.flat_dists)/4*part_stats.efforts/max(part_stats.efforts), 'm--')
    legend('heights (4 x scale)', 'accelerations (height normalized)', 'speed (height normalized)', 'effort (height normalized)', 'Location', 'SouthWest')
    grid on;
    hold off;
    
  end
  
end