function [X, fX] = minimize_Stoch(X, f, fCorrect, numiters, stoch_eps, varargin)

  CONSTANTS

  X = feval(fCorrect, X, varargin{:});
  [fX df] = feval(f, X, varargin{:});
%   [df]=findGrad(f,X, 1e-5, varargin{:});

  if PRINT_OUTPUT
    fprintf('%0.2f ', fX);
  end
  n_printed = 1;

  fs = zeros(numiters+1,1);
  fs(1) = fX;
  
  for i = 1:numiters

    if RENDER_PLOTS
      figure(101);
      if i == 1
        X_bak = X;
      end
      plot(X_bak, 'b--'); hold on;
      plot(X, 'b-'); hold off;
      ax = axis;
      axis([1, size(X,1), -max(abs(ax(3)), abs(ax(4))), max(abs(ax(3)), abs(ax(4)))]);
      drawnow;
      grid on;
    end
    
    fX_last = fX;
    X_last = X;
    
    X = X - stoch_eps.*df;
    X = feval(fCorrect, X, varargin{:});
    
    df_last = df;
    [fX df] = feval(f, X, varargin{:});
%     [df]=findGrad(f,X, 1e-5, varargin{:});

    if any(isnan(df)) || sqrt(sum(df.^2)./sum(df_last.^2))>10
      rand_scale = sqrt(sum(df_last.^2))/25;
      r = rand(size(X));
%       r = rand_scale*sign(2*r-1).*-log(abs(2*r-1));
      r = 2*rand_scale*(r-0.5);
      df = filter2(fspecial('Gaussian', 9, 1.5), r, 'same');
    end

    df = df + MINIMIZE_MOMENTUM*df_last;
    if fX > fX_last || any(isnan(df)) || isnan(fX)
      X = X_last;
      fX = fX_last;
      stoch_eps = stoch_eps*MINIMIZE_EPS_CHOP;
    else
      stoch_eps = stoch_eps*MINIMIZE_EPS_INC;
    end
        
    fs(i+1) = fX;

    if PRINT_OUTPUT && (mod(i, 10) == 0)
      fprintf('%0.2f ', fX);
      n_printed = n_printed + 1;
      if n_printed >= 10
        fprintf('\n');
        n_printed = 0;
      end
    end
    
    if i>(MINIMIZE_STOCH_STREAK_LENGTH+1) && all(abs(fs(i-MINIMIZE_STOCH_STREAK_LENGTH:i-1) - fs(i-(MINIMIZE_STOCH_STREAK_LENGTH-1):i)) < 0.01)
      break;
    end
    
  end

  if PRINT_OUTPUT
    fprintf('\n')
  end
  
end
