function [cost, D_cost, part_cost, part_stats] = pathCost(accelerations, biker, path, is_start)

  CONSTANTS

  if is_start  
    s_init = 0;
  else
    s_init = biker.stop_speed;
  end
  
  s_end = biker.stop_speed;
  
  distances = path.true_dists;
  grades = path.grades;
  len = length(accelerations);
  
  distances_mat = repmat(distances, 1, len);
  accelerations_mat = repmat(accelerations, 1, len);
  grade_mat = repmat(grades, 1, len);
  
  no_accel = abs(accelerations)<10^-8;
  no_accel_mat = repmat(no_accel, 1, len);
  
  speeds = sqrt(2)*sqrt(0.5*s_init^2 + cumsum(accelerations .* distances));
  speeds_before = [s_init; speeds(1:end-1)];
  speeds_before_mat = repmat(speeds_before, 1, len);
  
  D_speeds_mat = tril((1./speeds) * distances'); % D(Sn)/D(Am)
  D_speeds_mat_before = [zeros(1, len); D_speeds_mat(1:end-1,:)];
  
%   cost = sum(speeds(end));
%   D_cost = D_speeds_mat(end,:)';
  
%   cost = sum(speeds);
%   D_cost = sum(D_speeds_mat)';

  times = (speeds - speeds_before)./accelerations;
  times(no_accel) = distances(no_accel)./speeds_before(no_accel);
  
  D_times_mat = (D_speeds_mat - D_speeds_mat_before)./repmat(accelerations, 1, len);
  D_times_mat = D_times_mat + diag((speeds - speeds_before).*(-accelerations.^-2));
  
  D_times_mat_no_accel = tril(-distances_mat./(speeds_before_mat.^2).*D_speeds_mat);
  D_times_mat_no_accel = D_times_mat_no_accel + diag(distances./(speeds.^2).*diag(D_speeds_mat))/2;
  
  D_times_mat(no_accel_mat) = D_times_mat_no_accel(no_accel_mat);
  
  total_time = sum(times);
  D_times = sum(D_times_mat,1)';
    
%   cost = times(end);
%   D_cost = D_times_mat(end,:)';
  
%   cost = sum(times);
%   D_cost = sum(D_times_mat)';
    
  K1 = biker.K1;
  K2 = biker.K2;
  M = biker.mass;
  
  % Take the midpoint speed
  sm = (speeds + speeds_before)/2;
  sm_mat = repmat(sm, 1, len);
  D_sm_mat = 0.5*(D_speeds_mat + D_speeds_mat_before);

  % Take the final speed
%   sm = speeds;
%   sm_mat = repmat(sm, 1, len);
%   D_sm_mat = D_speeds_mat;

  effort_power = biker.effort_power;

  energy_rates = (sm .* (K1 + (K2 .* (sm.^2)) + (K3*M.*(grades + K4/GRAVITY .* accelerations))))/1000;
  D_energy_rates_mat = (D_sm_mat.*(accelerations_mat*K3*K4*M + GRAVITY*(K1 + grade_mat*K3*M + 3*K2*(sm_mat.^2))))/(1000*GRAVITY) + diag((K3*K4*M*sm)/(1000*GRAVITY));
    
  neg_energies = energy_rates < 0;
  energy_rates(neg_energies) = 0;
  D_energy_rates_mat(repmat(neg_energies, 1, len)) = 0;

  effort_rates = 1/effort_power * (energy_rates.^effort_power);
  D_effort_rates_mat = D_energy_rates_mat .* repmat(energy_rates.^(effort_power-1), 1, len);
  
  p = effort_power;
  enr = energy_rates(energy_rates>0);
  t = times(energy_rates>0);
    
  D_cost_power = biker.effort*sum(( (enr.^p).*t.*(-1 + p.*log(enr)))./(p^2));
    
  efforts = effort_rates .* times;
  D_efforts_mat = D_effort_rates_mat .* repmat(times, 1, len) + repmat(effort_rates, 1, len) .* D_times_mat;

  total_effort = sum(efforts);
  D_efforts = sum(D_efforts_mat(efforts>0,:),1)';
  
%   cost = total_energy;
%   D_cost = D_energies;
  
  % Use max energy
%   energy_max_idx = find(energies == max(energies), 1, 'first');
%   total_energy = energies(energy_max_idx);
%   D_energies = D_energies_mat(energy_max_idx,:)';

  irreg_distances = (distances(1:end-1) + distances(2:end))/2;
  
%   keyboard
  irregularities = ((accelerations(2:end) - accelerations(1:end-1))./irreg_distances);

%   irregularity = sum(abs(irregularities));
%   D_irregularity = [0; sign(irregularities)./irreg_distances] - [sign(irregularities)./irreg_distances; 0];
    
  irregularity = sum(0.5 * irregularities.^2);
  D_irregularity = [0; irregularities./irreg_distances] - [irregularities./irreg_distances; 0];
    
%   irregularity = sum(0.5*((accelerations(2:end) - accelerations(1:end-1))./distances).^2);
%   D_irregularity = [0; (accelerations(2:end) - accelerations(1:end-1)) ./ distances] + [(accelerations(1:end-1) - accelerations(2:end)) ./ distances; 0];

%   D_irregularity = (([0; (accelerations(2:end) - accelerations(1:end-1))] + [(accelerations(1:end-1) - accelerations(2:end)); 0]) .* path.true_dist) ./ (path.true_dist.^2);
  
%   irregularity = sum(0.5*(accelerations(2:end) - accelerations(1:end-1)).^2);
%   D_irregularity = [0; (accelerations(2:end) - accelerations(1:end-1))] + [(accelerations(1:end-1) - accelerations(2:end)); 0];
    
  end_stop = 0.5*(speeds(end)-s_end)^2;
  D_end_stop = (speeds(end)-s_end) .* D_speeds_mat(end,:)';
  
%   too_fast = max(0,speeds-biker.max_speed);
%   D_too_fast = sum(D_speeds_mat .* repmat(too_fast > 0, 1, len))';
  
  cost = ...
      biker.timing*total_time ...
    + biker.effort*total_effort ...
    + biker.regularity*irregularity ...
    + biker.end_stopping*end_stop;
%     + biker.max_speed_mult*sum(too_fast);
  D_cost = ...
      biker.timing*D_times ...
    + biker.effort*D_efforts ...
    + biker.regularity*D_irregularity ...
    + biker.end_stopping*D_end_stop;
%     + biker.max_speed_mult*D_too_fast;

  part_cost.effort = total_effort;
  part_cost.time = total_time;
  part_cost.irregularity = irregularity;
  part_cost.end_stop = end_stop;
  part_cost.D_cost_power = D_cost_power;
  
  
  part_stats.efforts = efforts;
  part_stats.speeds = speeds;
  part_stats.times = times;
  
  assert(~(isnan(cost) || any(isnan(D_cost)) || imag(cost) || any(imag(D_cost))))
  
end

  
%   energies = zeros(len,1);
%   D_energies_mat = zeros(len,len);
%   
%   for i = 1:len
%     
%     a = accelerations(i);
%     g = grades(i);
%     s1 = speeds(i);
%         
%     sm = s1;
%     energies(i) = (sm .* (K1 + (K2 * sm.^2) + (K3*M.*(g + K4/GRAVITY .* a))))/1000;
%     
%     for j = 1:i
%       D_sm = D_speeds_mat(i,j);
%       
%       D_energies_mat(i,j) = (D_sm*(a*K3*K4*M + GRAVITY*(K1 + g*K3*M + 3*K2*(sm^2))))/(1000*GRAVITY);
%       if i == j
%         D_energies_mat(i,j) = D_energies_mat(i,j) + (K3*K4*M*sm)/(1000*GRAVITY);
%       end
%       
%     end
%     
%   end
%   


%   for i = 1:len
%     
%     a = accelerations(i);
%     d = distances(i);
%     g = grades(i);
%     
% %     if a == 0
% %       s1 = s0;
% %       t = d/s0;
% %       
% %       D_t = 0;
% %       D_s1 = 0;
% % 
% %     else
% %       s1 = sqrt(s0^2+2*a*d);
% % %       s1 = speeds(i);
% %       t = (s1 - s0)/a;
% % 
% % %       D_s1 = D_speeds(i);
% %       D_s1 = (d + s0*D_s0)/s1;
% %       D_t = (s0 - a*D_s0 - s1 + a*D_s1)/(a^2);
% % 
% %     end
%     
% %     speeds(i) = s1;
% %     
% %     if isinf(t) || imag(s1)
% %       state.time = Inf;
% %       state.cost = Inf;
% %       cost = Inf;
% %       return
% %     end
%     
% %     times(i) = t;
% %     D_time(i) = D_t;
%         
%     K1 = biker.K1;
%     K2 = biker.K2;
%     M = biker.mass;
%     K3 = 10.32;
%     K4 = 1.01;
%         
%     sm = (s0 + s1)/2;
%     D_sm = 0.5*D_s1;
%         
%     energy = max(0, (sm .* (K1 + (K2 * sm.^2) + (K3*M.*(g + K4/GRAVITY .* max(0,a)))))/1000);
%     D_energy = (1/(1000*GRAVITY))*(D_t*sm*(a*K3*K4*M + GRAVITY*(K1 + g*K3*M + K2*(sm^2))) + K3*K4*M*sm*t + D_sm*(a*K3*K4*M + GRAVITY*(K1 + g*K3*M + 3*K2*(sm^2))));
% 
%     energies(i) = energy;
%     D_energies(i) = D_energy;
%     
%   end
%   
%   energy_max_idx = find(energies == max(energies), 1, 'first');
%   
%   state.energy = energies(energy_max_idx);
%   state.time = sum(times);
%   
% %   D_time = flipud(D_time);
% %   D_time = flipud(D_time + (cumsum(D_time) - D_time));
%   
%   D_energies(setdiff(1:len, energy_max_idx)) = 0;
%   
%   state.cost = state.time + biker.mixing*state.energy;
%   cost = state.cost;
%     
% %   cost = state.energy;
% %   D_cost = D_energies;
% 
%   cost = state.time;
%   D_cost = D_time;
%     
% end


% % This works, and minimizes total energy expended.
% function [cost, D_cost, state] = pathCost(accelerations, biker, path, state)
% 
%   CONSTANTS
% 
%   D_time = zeros(size(accelerations));
%   D_energy = zeros(size(accelerations));
%   D_cost = zeros(size(accelerations));
%     
%   s1 = 0;
%   
%   for i = 1:(path.n_samples-1)
%     
%     a = accelerations(i);
%     d = path.true_dist(i);
%     g = path.grade(i);
%     s0 = s1;
%     
%     a = max(0,a);
% 
%     if a == 0
%       s1 = s0;
%       t = d/s0;
%       
%       D_t = 0;
%       D_s1 = 0;
% 
%     else
%       s1 = sqrt(s0^2+2*a*d);
%       t = (s1 - s0)/a;
%       
%       D_t = (-a*d + s0*(s1 - s0))/((a^2)*s1);
%       D_s1 = D_t;
% 
%     end
%     
%     if isinf(t) || imag(s1)
%       state.time = Inf;
%       state.cost = Inf;
%       cost = Inf;
%       return
%     end
%     
%     state.time = state.time + t;
%     D_time(i) = D_t;
%     
%     K1 = biker.K1;
%     K2 = biker.K2;
%     M = biker.mass;
%     K3 = 10.32;
%     K4 = 1.01;
%         
%     sm = (s0 + s1)/2;
%     D_sm = 0.5*D_s1;
%         
%     if a == 0
%       E_rate = 0;
%       E = 0;
%       D_E = 0;
%     else
%       E_rate = max(0, (sm .* (K1 + (K2 * sm.^2) + (K3*M.*(g + K4/GRAVITY .* max(0,a)))))/1000);
%       E = t*E_rate;
%       D_E = (1/(1000*GRAVITY))*(D_t*sm*(a*K3*K4*M + GRAVITY*(K1 + g*K3*M + K2*(sm^2))) + K3*K4*M*sm*t + D_sm*(a*K3*K4*M + GRAVITY*(K1 + g*K3*M + 3*K2*(sm^2)))*t);
%     end
%     
%     state.energy = state.energy + E;
%     D_energy(i) = D_E;
% 
%   end
%     
%   D_time = flipud(D_time);
%   D_time = flipud(D_time + 2*(cumsum(D_time) - D_time));
% 
% %   D_energy = 1.4408 * D_energy;
% %   D_energy = flipud(D_energy);
% %   D_energy = flipud(D_energy + 0.4*(cumsum(D_energy) - D_energy));
% %   D_energy = D_energy + (3*[length(D_energy):-1:1]');
%   D_energy(accelerations>0) = D_energy(accelerations>0) + 3*[sum(accelerations>0):-1:1]';
% 
%   state.cost = state.time + biker.mixing*state.energy;
% 
%   cost = state.cost;
%   D_cost = D_time + biker.mixing*D_energy;
% 
% end


% function [cost, D_cost, state] = pathCost(accelerations, biker, path, state)
% 
%   CONSTANTS
% 
%   D_cost = NaN*ones(size(accelerations));;
%   
%   D_s1 = 0;
%   s1 = 0;
%   
%   for i = 1:(path.n_samples-1)
%     
%     a = accelerations(i);
%     d = path.true_dist(i);
%     g = path.grade(i);
%     s0 = s1;
%     D_s0 = D_s1;
%     
%     % true_dist(1) = speeds(1) * t + 0.5*accelerations(1)*t.^2;
%     % 0 = 0.5*accelerations(1)*t.^2 + speeds(1)*t - true_dist(1);
%     
%     % length of time
%     if accelerations(i) == 0
%       t = d/s;
%     else
%       t = (-s0 + sqrt(s0.^2 + 2*a*d))/a;
% %       D_t = (s0 - (a*d + s0^2)/sqrt(2*a*d + s0^2))/(a^2);
% 
% %       D_t = 1/( (a^2) * sqrt(2*a*d + s0^2)) * (-a*d + s0*(-s0 + sqrt(2*a*d + s0^2)) + a*(s0-sqrt(2*a*d + s0^2)*D_s0));
% 
%       D_t = (-a*d + (-a*D_s0 + s0)*(-s0 + sqrt(2*a*d + s0^2)))/(a^2 * sqrt(2*a*d + s0^2));
%     end
%     
%     if isinf(t) || (imag(t) ~= 0)
%       % fail!
%       state.speed = 0;
%       state.time = Inf;
%       state.cost = Inf;
%       return;
%     end
%     
%     s1 = s0 + t * a;
% %     D_s1 = d/sqrt(2*a*d + s0^2);
%     D_s1 = (2*d + 2*D_s0*s0)/(2*sqrt(2*a*d + s0^2));
% 
% %     D_s1 = (d + s0*D_s0)/sqrt(2*a*d + s0^2);
% 
%     state.cost = state.cost + t;
%     D_cost(i) = D_t;
%     
%     % For speed in energy, use (s0+s1)/2. Average it out.
%     
% %     % kilojoules / second
% %     K1 = biker.K1;
% %     K2 = biker.K2;
% %     M = biker.mass;
% %     K3 = 10.32;
% %     K4 = 1.01;
% %     dE = (s .* (K1 + (K2 * s.^2) + (K3*M.*(g + K4/GRAVITY .* max(0,a)))))/1000;
% %     D_dE = (6*a*d*d*GRAVITY*K2 + K3*K4*M*s0^2 + d*(3*a*K3*K4*M + GRAVITY*(K1 + g*K3*M + 3*K2*s0^2)))/(1000*GRAVITY*sqrt(2*a*d + s0^2));
% %     
% %     state.speed = s;
% %     
% %     state.time = state.time + t;
% %     state.energy = state.energy + dE * t;
% % 
% %     D_time = D_t;
% %     D_energy = dE * D_t + D_dE * t;
% %     
% %     state.cost = state.cost + t * (1 + biker.mixing*dE);
% %     D_cost(i) = (1+biker.mixing*dE)*D_t + biker.mixing*t*D_dE;
%     
%   end
%   
%   cost = state.cost;
%   
% end


% function [cost, D_cost, state] = pathCost(accelerations, biker, path, state)
% 
%   CONSTANTS
% 
%   D_cost = NaN*ones(size(accelerations));;
%   
%   for i = 1:(path.n_samples-1)
%     
%     a = accelerations(i);
%     d = path.true_dist(i);
%     g = path.grade(i);
%     s = state.speed;
%     s0 = s;
%     
%     % true_dist(1) = speeds(1) * t + 0.5*accelerations(1)*t.^2;
%     % 0 = 0.5*accelerations(1)*t.^2 + speeds(1)*t - true_dist(1);
%     
%     % length of time
%     if accelerations(i) == 0
%       t = d/s;
%     else
%       t = (-s + sqrt(s.^2 + 2*a*d))/a;
%       D_t = (-a*d + s*(-s + sqrt(2*a*d*s*s)))/(a*a*sqrt(2*a*d+s*s));
%     end
%     
%     if isinf(t) || (imag(t) ~= 0)
%       % fail!
%       state.speed = 0;
%       state.time = Inf;
%       state.cost = Inf;
%       return;
%     end
%     
%     D_s = d/sqrt(2*a*d+s^2); % deriv over a
%     s = s + t * a;
% 
%     % kilojoules / second
%     K1 = biker.K1;
%     K2 = biker.K2;
%     M = biker.mass;
%     K3 = 10.32;
%     K4 = 1.01;
%     dE = (s .* (K1 + (K2 * s.^2) + (K3*M.*(g + K4/GRAVITY .* max(0,a)))))/1000;
%     D_dE = (6*a*d*d*GRAVITY*K2 + K3*K4*M*s0^2 + d*(3*a*K3*K4*M + GRAVITY*(K1 + g*K3*M + 3*K2*s0^2)))/(1000*GRAVITY*sqrt(2*a*d + s0^2));
%     
%     state.speed = s;
%     
%     state.time = state.time + t;
%     state.energy = state.energy + dE * t;
% 
%     D_time = D_t;
%     D_energy = dE * D_t + D_dE * t;
%     
%     state.cost = state.cost + t * (1 + biker.mixing*dE);
% 
%     D_cost(i) = (1+biker.mixing*dE)*D_t + biker.mixing*t*D_dE;
%     
%   end
%   
%   cost = state.cost;
%   
% end
