function [shading] = lossfun_likelihood(height_point, L_point)

height = point2grid(height_point);

[im_i, im_j] = find(ones(size(height_point)));
data.rendering.idx_self    = sub2ind(size(height), im_i,     im_j    );
data.rendering.idx_neigh1  = sub2ind(size(height), im_i + 1, im_j    );
data.rendering.idx_neigh2  = sub2ind(size(height), im_i,     im_j + 1);
data.rendering.idx_neigh12 = sub2ind(size(height), im_i + 1, im_j + 1);

% Get the heights of horizontal/vertical neighboring pixels
h   = height(data.rendering.idx_self);
hx  = height(data.rendering.idx_neigh1);
hy  = height(data.rendering.idx_neigh2);
hxy = height(data.rendering.idx_neigh12);
 
% Compute the normal from the difference in heights (approximates the partial derivative)
n1 = [h - hx,   h - hy,   ones(size(h))];
n2 = [hy - hxy, hx - hxy, ones(size(h))];

% Normalize the normals
n = ( ...
  n1 ./ repmat(sqrt(sum(n1.^2,2)), 1, size(n1,2)) + ...
  n2 ./ repmat(sqrt(sum(n2.^2,2)), 1, size(n2,2)) ...
  )/2;

% Assert that the light direction is normalized
% assert(abs(sqrt(sum(L.^2)) - 1) < 10^-10);
% L = L ./ sqrt(sum(L.^2));

L = -[(im_i - L_point(1)), (im_j - L_point(2)), h - L_point(3)];
L = L ./ repmat(sqrt(sum(L.^2,2)), 1, size(L,2));

% Compute the dot product of light & normal
s = max(0, sum(n .* L,2));

shading = reshape(s, size(height)-1);


