function [nodes_interp, nodes_internal, tris] = tesselate(nodes, edges)

LEN = 0.02;

nodes_interp = {};
edge_children = {};

count = 0;
for i = 1:size(edges,1)

  p1 = nodes(edges(i,1),:);
  p2 = nodes(edges(i,2),:);
  
  v = (p2 - p1);
  v_mag = sqrt(sum(v.^2));
  n = max(1, floor(v_mag/LEN));
  nodes_interp{i} = repmat([0:n]', 1, 2) .* repmat(v / n, n+1, 1) + repmat(p1, n+1, 1);
  
  assert(~(any(isnan(nodes_interp{i}(:))) || any(isinf(nodes_interp{i}(:)))))

%   s = [0:n]'/n;
%   [1-s, s] * [p1;p2]
% 
%   keyboard
  
  edge_children{i} = count + [1:size(nodes_interp{i},1)]';
  count = count + size(nodes_interp{i},1);
  
  edges_interp{i} = [edge_children{i}(1:end-1), edge_children{i}(2:end)];
  
end

nodes_interp = cat(1, nodes_interp{:});

[nodes_interp_u] = unique(nodes_interp, 'rows');

for i = 1:size(edges,1)
  idx = edge_children{i};
  [tf, loc1] = ismember(nodes_interp(idx,:), nodes_interp_u, 'rows');
  edge_children{i} = loc1;
    
  [tf, loc2] = ismember(edges_interp{i}, idx);
  edges_interp{i} = reshape(loc1(loc2), size(edges_interp{i}));
  
end

nodes_interp = nodes_interp_u;


% for i = 1:size(edges,1)
%   clf;
%   line([nodes_interp(edges_interp{i}(:,1),1), nodes_interp(edges_interp{i}(:,2),1)]', [nodes_interp(edges_interp{i}(:,1),2), nodes_interp(edges_interp{i}(:,2),2)]', 'Color', 'y', 'LineWidth', 2);
%   hold on;
%   scatter(nodes(edges(i,:),1), nodes(edges(i,:),2), 'ro', 'filled')
%   scatter(nodes_interp(edge_children{i},1), nodes_interp(edge_children{i},2), 100, 'bx')
%   drawnow;
%   pause(0.5)
% end

x = [LEN:LEN:1-LEN]';
nodes_boundary = [[x, 0*x]; [0*x, x]; [x, 0*x+1]; [0*x+1, x]];
nodes_all = [nodes_interp; nodes_boundary];
nodes_all_unique = unique(nodes_all, 'rows');

assert(~(any(isnan(nodes_all(:))) || any(isinf(nodes_all(:)))))


V = nodes; E = edges;
line([V(E(:,1),1), V(E(:,2),1)]', [V(E(:,1),2), V(E(:,2),2)]', 'Color', 'r', 'LineWidth', 3)
hold on;
voronoi(nodes_all_unique(:,1), nodes_all_unique(:,2))
axis image off;
% print('-dpng', ['voronoi.png']);


[vx, vy] = voronoi(nodes_all_unique(:,1), nodes_all_unique(:,2));
% clear nodes_all

v_nodes = unique([vx(:),vy(:)], 'rows');
v_nodes = v_nodes(all(v_nodes >= 0 & v_nodes <= 1,2),:);

% keyboard
D = distMat(v_nodes, nodes_all);
D = D - repmat(min(D, [], 2), 1, size(D,2));
D = D(:,(size(nodes_interp,1)+1):end);

remove = sum(D <= 10^-12, 2)>2;
v_nodes = v_nodes(~remove,:);


[tf1, loc1] = ismember([vx(1,:)', vy(1,:)'], v_nodes, 'rows');
[tf2, loc2] = ismember([vx(2,:)', vy(2,:)'], v_nodes, 'rows');

v_edges = [loc1(tf1 & tf2), loc2(tf1 & tf2)];

does_intersect = LineSegmentIntersect(nodes, v_nodes, edges, v_edges);
v_edges = v_edges(setdiff(1:size(v_edges,1), does_intersect(:,2)),:);


line([v_nodes(v_edges(:,1),1), v_nodes(v_edges(:,2),1)]', [v_nodes(v_edges(:,1),2), v_nodes(v_edges(:,2),2)]', 'Color', 'r');
hold on;
scatter(v_nodes(:,1), v_nodes(:,2), 10, 'ro', 'MarkerFaceColor', [0,0,0])

line([nodes(edges(:,1),1), nodes(edges(:,2),1)]', [nodes(edges(:,1),2), nodes(edges(:,2),2)]', 'Color', 'k');
hold on;
scatter(nodes(:,1), nodes(:,2), 20, 'ko', 'MarkerFaceColor', [0,0,0])
axis([0,1,0,1])

D = distMat(nodes_interp, v_nodes);
D = D - repmat(min(D, [], 1), size(D,1),1);
[D_i,D_j] = find(D <= 10^-8);

nodes_all = [nodes_interp; v_nodes];
nodes_label = [0*nodes_interp(:,1); 1+0*v_nodes(:,1)]; % 1 == V
nodes_all_v_offset = size(nodes_interp(:,1),1);

edges_all = [cat(1,edges_interp{:}); v_edges + nodes_all_v_offset];
edges_all_mat = sparse(edges_all(:,1), edges_all(:,2), ones(size(edges_all,1),1), size(nodes_all,1), size(nodes_all,1));
edges_all_mat = edges_all_mat | edges_all_mat';

edge_tris = {};

for ei = 1:size(edges,1)
  
  loc = edge_children{ei};

  tf = ismember(D_i, loc);
  ii = D_i(tf);
  jj = D_j(tf);

  % Remove children who are on the other side of neighboring edges
  endpoints = edges(ei,:);
  for ee = 1:2
    n = [edges(edges(:,1) == endpoints(ee),2); edges(edges(:,2) == endpoints(ee),1)];
    n0 = endpoints(3-ee);
    n = setdiff(n, n0);
    v = nodes(n,:) - repmat(nodes(endpoints(ee),:), length(n), 1);
    v0 = nodes(n0,:) - nodes(endpoints(ee),:);
    
    th = cart2pol( [v0(1); v(:,1)], [v0(2); v(:,2)]);
    th0 = th(1);
    dth = mod(th(2:end) - th(1), 2*pi);
    sorted = sort(dth, 'ascend');

    if length(sorted) < 2
      continue
    end
    dth_range = sorted([1,end]);
    
    
    v_neighbors = v_nodes(jj,:);
    v = v_neighbors - repmat(nodes(endpoints(ee),:), size(v_neighbors,1), 1);
    th = cart2pol( [v(:,1)], [v(:,2)]);
    dth = mod(th - th0, 2*pi);
    
    keep = (dth <= dth_range(1)) | (dth >= dth_range(2));
    jj = jj(keep);
    ii = ii(keep);
  end
  
 
  
%   clf
%   line([v_nodes(v_edges(:,1),1), v_nodes(v_edges(:,2),1)]', [v_nodes(v_edges(:,1),2), v_nodes(v_edges(:,2),2)]', 'Color', 'r');
%   hold on;
%   scatter(v_nodes(:,1), v_nodes(:,2), 10, 'ro', 'MarkerFaceColor', [0,0,0])
%   line([nodes(edges(:,1),1), nodes(edges(:,2),1)]', [nodes(edges(:,1),2), nodes(edges(:,2),2)]', 'Color', 'k');
%   hold on;
%   scatter(nodes(:,1), nodes(:,2), 20, 'ko', 'MarkerFaceColor', [0,0,0])
%   axis([0,1,0,1])
% 
%   plot(nodes_interp(ii,1), nodes_interp(ii,2), 'LineWidth', 3)  
%   hold on;
%   line([nodes_interp(ii,1), v_nodes(jj,1)]', [nodes_interp(ii,2), v_nodes(jj,2)]', 'Color', 'r', 'LineWidth', 2);
%   scatter(v_nodes(jj,1), v_nodes(jj,2), 50, 'ro', 'MarkerFaceColor', [0,0,0])
% 
%   drawnow;

  
  
  jj_o = nodes_all_v_offset + jj;
  
  tris = {};
  for i = 1:size(ii,1)
    jj_match = jj_o(i,:) == jj_o;
    jj_match(i) = 0;
    
    ii_match = ii(i,:) == ii;
    ii_match(i) = 0;
    
    tris{length(tris)+1} = [jj_o(ii_match), repmat(ii(i), sum(ii_match),1), repmat(jj_o(i), sum(ii_match),1)];
        
    tris{length(tris)+1} = [repmat(ii(i), sum(jj_match),1), repmat(jj_o(i), sum(jj_match),1), ii(jj_match)];
    
  end
  
  tris = unique(cat(1,tris{:}), 'rows');
  
  complete = full(edges_all_mat(sub2ind(size(edges_all_mat), tris(:,1), tris(:,3))));
  tris = tris(complete,:);
  
  edge_tris{ei} = tris;
  
%   clf
%   
%   patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', repmat(rand(size(tris,1),1), [1,3])', 'EdgeColor', 'none', 'FaceAlpha', .25)
%   hold on;
%   scatter(nodes_interp(ii,1), nodes_interp(ii,2), 50, 'bo')
%   scatter(v_nodes(jj,1), v_nodes(jj,2), 50, 'ro')
%   
%   pause
  
end


tris = cat(1,edge_tris{:});

% scatter(nodes_interp(23,1), nodes_interp(23,2), 500, 'rx')
% scatter(nodes_interp(27,1), nodes_interp(27,2), 500, 'rx')

v_nodes_dupe = {};
for ti = 1:size(tris,1)
  
  tri = tris(ti,:);
  
  is_v = find(tri > nodes_all_v_offset);

  for i = is_v
    c = length(v_nodes_dupe)+1;
    n = nodes_all(tri(i),:);
    v_nodes_dupe{c} = n;
    tris(ti,i) = c + nodes_all_v_offset;
  end
    
end

v_nodes = cat(1,v_nodes_dupe{:});



union_data = {};

for vi = 1:size(v_nodes,1)
  
  vio = vi + nodes_all_v_offset;

  v_match = find(all(v_nodes == repmat(v_nodes(vi,:), size(v_nodes,1), 1),2)) + nodes_all_v_offset;

  t_match = find(any(ismember(tris, v_match),2));

  t0_match = find(any(tris == vio,2));
  t_match = setdiff(t_match, t0_match);

  neighbors = setdiff(tris(t0_match,:), vio);
  neighbors = neighbors(neighbors <= nodes_all_v_offset);

  t_neighbor_match = t_match(any(ismember(tris(t_match,:), neighbors),2),:)';

  v_neighbor_match = intersect(reshape(tris(t_neighbor_match,:), [], 1), v_match);
  
  union_data{vi} = [repmat(vio, length(v_neighbor_match), 1), v_neighbor_match];
  
end

union_data = cat(1,union_data{:});

mat = sparse(union_data(:,1), union_data(:,2), ones(size(union_data(:,1))), size(v_nodes,1) + nodes_all_v_offset, size(v_nodes,1) + nodes_all_v_offset);
mat = double(mat | speye(size(mat)));

n_last = inf;
n = nnz(mat);
while n ~= n_last
  mat = double((mat * mat)>0);
  mat = double(mat | mat');
  n_last = n;
  n = nnz(mat);
end

for vi = nodes_all_v_offset + [1:size(v_nodes,1)]
  idx = find(mat(:,vi));
  tris(ismember(tris, idx)) = idx(1);
end


[u, ui, uj] = unique(tris(tris > nodes_all_v_offset));

tris(tris > nodes_all_v_offset) = uj+nodes_all_v_offset;
v_nodes = v_nodes(u-nodes_all_v_offset,:);


v_nodes_shrunk = nan(size(v_nodes));


for vi = 1:size(v_nodes,1)
  vi_o = vi + nodes_all_v_offset;
  
  sub_tris = tris(any(tris == vi_o,2),:);
  neighbors = unique(sub_tris(sub_tris < nodes_all_v_offset));
  
  v_nodes_shrunk(vi,:) = (mean(nodes_interp(neighbors,:),1) + v_nodes(vi,:))/2;
  
end

tris = unique(tris, 'rows');
patch_color = repmat(rand(size(tris,1),1), [1,3])';

% figure;
% line([nodes(edges(:,1),1), nodes(edges(:,2),1)]', [nodes(edges(:,1),2), nodes(edges(:,2),2)]', 'Color', 'k');
% hold on;
% scatter(nodes(:,1), nodes(:,2), 20, 'ko', 'MarkerFaceColor', [0,0,0])
% axis([0,1,0,1])
% 
% patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', patch_color, 'EdgeColor', 'none', 'FaceAlpha', .25)
% drawnow

nodes_internal = v_nodes;

swap = tris(:,3) < tris(:,1);
tris(swap,[1,3]) = tris(swap,[3,1]);
tris = unique(tris, 'rows');

nodes_all = [nodes_interp; nodes_internal];

patch_color = repmat(rand(size(tris,1),1), [1,3])';
patch([nodes_all(tris(:,1),1), nodes_all(tris(:,2),1), nodes_all(tris(:,3),1)]', [nodes_all(tris(:,1),2), nodes_all(tris(:,2),2), nodes_all(tris(:,3),2)]', patch_color, 'EdgeColor', 'none', 'FaceAlpha', .25)
drawnow

axis image off ij

