function triLay(data, grandparent, parent)

TRILENGTH_METHOD = 2;

global nodes_warp nodes_fencepost nodes_triangles

kids = data.children{parent};
if isempty(kids)
  return
end

amounts = data.subtree_amounts(kids);
if isempty(amounts)
  amounts = 1;
end

lengths = data.subtree_lengths(kids);

assert(max(data.costs) == 1)

if isempty(grandparent)

  p = nodes_warp(parent,:);
  
  theta_fenceposts = [0, cumsum(amounts) ./ sum(amounts)] * 2*pi;
  theta_centers = (theta_fenceposts(1:end-1) + theta_fenceposts(2:end))/2;

  [x,y] = pol2cart(theta_centers, data.costs(kids)');
  nodes_warp(kids,:) = [x;y]';

  for i = 1:length(kids)
    [x,y] = pol2cart(theta_fenceposts(i + [0,1]), [1,1]);
    fp = [x;y];
    nodes_fencepost{kids(i)}{1} = fp(:,1)';
    nodes_fencepost{kids(i)}{2} = fp(:,2)';
    nodes_triangles{kids(i)} = [fp'; p];
  end
  
else

  p = nodes_warp(parent,:);
  gp = nodes_warp(grandparent,:);
  
  pL = nodes_fencepost{parent}{1} - p;
  pR = nodes_fencepost{parent}{2} - p;
  sub_radius1 = sqrt(sum(pL.^2,2));
  sub_radius2 = sqrt(sum(pR.^2,2));
  assert(abs(sub_radius1 - sub_radius2) < 10^-6)
  sub_radius = sub_radius1;

  [theta_range] = mod(cart2pol([pL(1), pR(1)], [pL(2), pR(2)]), 2*pi);

  frac = [0, cumsum(amounts) ./ sum(amounts)];

  if theta_range(2) < theta_range(1)
    theta_range(2) = theta_range(2) + 2*pi;
  end

  theta_fenceposts = frac * (theta_range(2)-theta_range(1)) + theta_range(1);
  theta_centers = (theta_fenceposts(1:end-1) + theta_fenceposts(2:end))/2;

  for i = 1:length(kids)
      
    [a,b] = pol2cart(theta_centers(i), 1);
    [junk, nodes_warp(kids(i),:)] = quadFun(p, [a,b], data.costs(kids(i)));
    
    if TRILENGTH_METHOD == 2

      [d_max, p_max] = quadFun(p, [a,b], data.costs(parent) + data.subtree_lengths(kids(i)));

      for j = 1:2
        [x,y] = pol2cart(theta_fenceposts(i+j-1), 1);
        a = p_max(1) - p(1); b = p_max(2) - p(2);
        q = (a.^2 + b.^2)./(a.*x + b.*y);
        nodes_fencepost{kids(i)}{j} = (q*[x,y] + p);
      end

      nodes_triangles{kids(i)} = [cat(1,nodes_fencepost{kids(i)}{:}); p];

    elseif TRILENGTH_METHOD == 1

      % Old Triangle length assignment method
      [x,y] = pol2cart(theta_fenceposts(i + [0,1]), sub_radius*[1,1]);
      fp = [x;y] + repmat(p', 1, length(x));

      nodes_fencepost{kids(i)}{1} = fp(:,1)';
      nodes_fencepost{kids(i)}{2} = fp(:,2)';
      nodes_triangles{kids(i)} = [fp'; p];

    end

  end
  
%   if ~isempty(grandparent)
%     V_kids = cat(1,nodes_triangles{kids});
% 
%     e = [1, 1; 1, 2; 1,3; 2,3];
%     es = {};
%     for i = 1:length(kids)
%       es{i} = e + 3*(i-1);
%     end
%     E_kids = cat(1,es{:});
% 
%     V_parent = nodes_triangles{parent};
%     E_parent = e;
% 
%     I = LineSegmentIntersect(V_kids, V_parent, E_kids, E_parent);
% 
%     assert(size(I,1) == 0);
%   end
  
end

% keyboard
% clf
% patches = cat(3,nodes_triangles{data.traversal});
% ds = data.node_depths(data.traversal(1:size(patches,3)));
% patch(squeeze(patches(:,1,:)), squeeze(patches(:,2,:)), ds' ./ max(data.node_depths), 'EdgeColor', 'w')
% cmap = colormap('hsv');
% colormap(hsv2rgb(rgb2hsv(cmap) .* repmat([1, 0.5, 1], size(cmap,1), 1)))
% renderTree(nodes_warp, data.start_node_idx, [], data.edge_idx, 1);
% drawnow

for kid = kids
  triLay(data, parent, kid);
end

