function im_warp = warpImage(im, nodes_all, nodes_all_warp, tris, N_BINS)

range1 = [min(nodes_all_warp(:,1)), max(nodes_all_warp(:,1))];
range2 = [min(nodes_all_warp(:,2)), max(nodes_all_warp(:,2))];

idx1 = range1(1) : (range1(2) - range1(1))/(N_BINS) : range1(2);
idx1 = (idx1(2:end) + idx1(1:end-1))/2;

idx2 = range2(1) : (range2(2) - range2(1))/(N_BINS) : range2(2);
idx2 = (idx2(2:end) + idx2(1:end-1))/2;

[x,y] = meshgrid(idx1, idx2);
p = [x(:), y(:)];

  R_warp = nan(N_BINS, N_BINS);
  G_warp = nan(N_BINS, N_BINS);
  B_warp = nan(N_BINS, N_BINS);

  for ti = 1:size(tris,1)

    a = nodes_all_warp(tris(ti,1),:);
    b = nodes_all_warp(tris(ti,2),:);
    c = nodes_all_warp(tris(ti,3),:);

    inside = pointInTriangle(p, a,b,c);

    if ~(any(inside))
      continue
    end

    t1 = nodes_all(tris(ti,:),:)';
    t2 = nodes_all_warp(tris(ti,:),:)';

    if any(any(isnan(nodes_all_warp(tris(ti,:),:))))
      continue
    end

    [W, flag] = affineWarp(t2, t1);

    if flag
      continue
    end

    p_warp = (W * [p(inside,:)'; ones(1, sum(inside))])';

    p_warp_idx = p_warp .* repmat([size(im,1), size(im,2)], size(p_warp,1), 1);

    r = interp2(im(:,:,1), p_warp(:,1)*size(im,1), p_warp(:,2)*size(im,2));
    g = interp2(im(:,:,2), p_warp(:,1)*size(im,1), p_warp(:,2)*size(im,2));
    b = interp2(im(:,:,3), p_warp(:,1)*size(im,1), p_warp(:,2)*size(im,2));

    R_warp(inside) = r;
    G_warp(inside) = g;
    B_warp(inside) = b;

  end

  im_warp = cat(3, R_warp, G_warp, B_warp);
  im_warp(isnan(im_warp)) = 1;

